# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'src/leap/bitmask/gui/ui/eip_status.ui'
#
# Created: Wed Oct 30 17:39:24 2013
#      by: pyside-uic 0.2.15 running on PySide 1.2.1
#
# WARNING! All changes made in this file will be lost!

from PySide import QtCore, QtGui

class Ui_EIPStatus(object):
    def setupUi(self, EIPStatus):
        EIPStatus.setObjectName("EIPStatus")
        EIPStatus.resize(470, 157)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Preferred, QtGui.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(EIPStatus.sizePolicy().hasHeightForWidth())
        EIPStatus.setSizePolicy(sizePolicy)
        self.verticalLayout = QtGui.QVBoxLayout(EIPStatus)
        self.verticalLayout.setContentsMargins(-1, -1, -1, 24)
        self.verticalLayout.setObjectName("verticalLayout")
        self.gridLayout = QtGui.QGridLayout()
        self.gridLayout.setObjectName("gridLayout")
        self.btnEipStartStop = QtGui.QPushButton(EIPStatus)
        self.btnEipStartStop.setObjectName("btnEipStartStop")
        self.gridLayout.addWidget(self.btnEipStartStop, 0, 2, 1, 1)
        self.label = QtGui.QLabel(EIPStatus)
        self.label.setMaximumSize(QtCore.QSize(32, 32))
        self.label.setText("")
        self.label.setPixmap(QtGui.QPixmap(":/images/black/32/earth.png"))
        self.label.setObjectName("label")
        self.gridLayout.addWidget(self.label, 0, 0, 1, 1)
        self.lblEIPStatus = QtGui.QLabel(EIPStatus)
        self.lblEIPStatus.setMaximumSize(QtCore.QSize(16777215, 32))
        self.lblEIPStatus.setStyleSheet("color: rgb(80, 80, 80);")
        self.lblEIPStatus.setWordWrap(True)
        self.lblEIPStatus.setObjectName("lblEIPStatus")
        self.gridLayout.addWidget(self.lblEIPStatus, 3, 1, 1, 1)
        self.lblEIPMessage = QtGui.QLabel(EIPStatus)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Preferred, QtGui.QSizePolicy.Minimum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.lblEIPMessage.sizePolicy().hasHeightForWidth())
        self.lblEIPMessage.setSizePolicy(sizePolicy)
        font = QtGui.QFont()
        font.setPointSize(14)
        font.setWeight(75)
        font.setBold(True)
        self.lblEIPMessage.setFont(font)
        self.lblEIPMessage.setWordWrap(True)
        self.lblEIPMessage.setObjectName("lblEIPMessage")
        self.gridLayout.addWidget(self.lblEIPMessage, 0, 1, 1, 1)
        self.lblVPNStatusIcon = QtGui.QLabel(EIPStatus)
        self.lblVPNStatusIcon.setMaximumSize(QtCore.QSize(24, 24))
        self.lblVPNStatusIcon.setText("")
        self.lblVPNStatusIcon.setPixmap(QtGui.QPixmap(":/images/black/32/off.png"))
        self.lblVPNStatusIcon.setScaledContents(True)
        self.lblVPNStatusIcon.setObjectName("lblVPNStatusIcon")
        self.gridLayout.addWidget(self.lblVPNStatusIcon, 0, 3, 1, 1)
        spacerItem = QtGui.QSpacerItem(40, 0, QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Minimum)
        self.gridLayout.addItem(spacerItem, 1, 1, 1, 1)
        self.eip_bandwidth = QtGui.QWidget(EIPStatus)
        self.eip_bandwidth.setMaximumSize(QtCore.QSize(16777215, 32))
        self.eip_bandwidth.setObjectName("eip_bandwidth")
        self.horizontalLayout = QtGui.QHBoxLayout(self.eip_bandwidth)
        self.horizontalLayout.setSpacing(0)
        self.horizontalLayout.setContentsMargins(0, 0, 0, 0)
        self.horizontalLayout.setContentsMargins(0, 0, 0, 0)
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.eip_bandwidth_layout = QtGui.QHBoxLayout()
        self.eip_bandwidth_layout.setSpacing(4)
        self.eip_bandwidth_layout.setSizeConstraint(QtGui.QLayout.SetDefaultConstraint)
        self.eip_bandwidth_layout.setObjectName("eip_bandwidth_layout")
        self.label_5 = QtGui.QLabel(self.eip_bandwidth)
        self.label_5.setText("")
        self.label_5.setPixmap(QtGui.QPixmap(":/images/light/16/down-arrow.png"))
        self.label_5.setObjectName("label_5")
        self.eip_bandwidth_layout.addWidget(self.label_5)
        self.btnDownload = QtGui.QPushButton(self.eip_bandwidth)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Maximum, QtGui.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.btnDownload.sizePolicy().hasHeightForWidth())
        self.btnDownload.setSizePolicy(sizePolicy)
        self.btnDownload.setMinimumSize(QtCore.QSize(100, 0))
        self.btnDownload.setMaximumSize(QtCore.QSize(120, 16777215))
        font = QtGui.QFont()
        font.setPointSize(11)
        font.setWeight(75)
        font.setBold(True)
        self.btnDownload.setFont(font)
        self.btnDownload.setCursor(QtCore.Qt.PointingHandCursor)
        self.btnDownload.setStyleSheet("text-align: left;")
        self.btnDownload.setFlat(True)
        self.btnDownload.setObjectName("btnDownload")
        self.eip_bandwidth_layout.addWidget(self.btnDownload)
        spacerItem1 = QtGui.QSpacerItem(10, 20, QtGui.QSizePolicy.Fixed, QtGui.QSizePolicy.Minimum)
        self.eip_bandwidth_layout.addItem(spacerItem1)
        self.label_7 = QtGui.QLabel(self.eip_bandwidth)
        self.label_7.setText("")
        self.label_7.setPixmap(QtGui.QPixmap(":/images/light/16/up-arrow.png"))
        self.label_7.setObjectName("label_7")
        self.eip_bandwidth_layout.addWidget(self.label_7)
        self.btnUpload = QtGui.QPushButton(self.eip_bandwidth)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Maximum, QtGui.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.btnUpload.sizePolicy().hasHeightForWidth())
        self.btnUpload.setSizePolicy(sizePolicy)
        self.btnUpload.setMinimumSize(QtCore.QSize(100, 0))
        self.btnUpload.setMaximumSize(QtCore.QSize(120, 16777215))
        font = QtGui.QFont()
        font.setPointSize(11)
        font.setWeight(75)
        font.setBold(True)
        self.btnUpload.setFont(font)
        self.btnUpload.setCursor(QtCore.Qt.PointingHandCursor)
        self.btnUpload.setStyleSheet("text-align: left;")
        self.btnUpload.setFlat(True)
        self.btnUpload.setObjectName("btnUpload")
        self.eip_bandwidth_layout.addWidget(self.btnUpload)
        spacerItem2 = QtGui.QSpacerItem(0, 20, QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Minimum)
        self.eip_bandwidth_layout.addItem(spacerItem2)
        self.horizontalLayout.addLayout(self.eip_bandwidth_layout)
        self.gridLayout.addWidget(self.eip_bandwidth, 2, 1, 1, 3)
        self.verticalLayout.addLayout(self.gridLayout)

        self.retranslateUi(EIPStatus)
        QtCore.QMetaObject.connectSlotsByName(EIPStatus)

    def retranslateUi(self, EIPStatus):
        EIPStatus.setWindowTitle(QtGui.QApplication.translate("EIPStatus", "Form", None, QtGui.QApplication.UnicodeUTF8))
        self.btnEipStartStop.setText(QtGui.QApplication.translate("EIPStatus", "Turn On", None, QtGui.QApplication.UnicodeUTF8))
        self.lblEIPStatus.setText(QtGui.QApplication.translate("EIPStatus", "...", None, QtGui.QApplication.UnicodeUTF8))
        self.lblEIPMessage.setText(QtGui.QApplication.translate("EIPStatus", "Traffic is being routed in the clear", None, QtGui.QApplication.UnicodeUTF8))
        self.btnDownload.setText(QtGui.QApplication.translate("EIPStatus", "0.0 KB/s", None, QtGui.QApplication.UnicodeUTF8))
        self.btnUpload.setText(QtGui.QApplication.translate("EIPStatus", "0.0 KB/s", None, QtGui.QApplication.UnicodeUTF8))

import mainwindow_rc
import icons_rc



# -*- coding: utf-8 -*-
# test_srpregister.py
# Copyright (C) 2013 LEAP
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""
Tests for:
    * leap/crypto/srpregister.py
"""
try:
    import unittest2 as unittest
except ImportError:
    import unittest
import os
import sys

from mock import MagicMock
from nose.twistedtools import reactor, deferred
from twisted.python import log
from twisted.internet import threads

from leap.bitmask.config.providerconfig import ProviderConfig
from leap.bitmask.crypto import srpregister, srpauth
from leap.bitmask.crypto.tests import fake_provider
from leap.common.testing.https_server import where

log.startLogging(sys.stdout)


def _get_capath():
    return where("cacert.pem")

_here = os.path.split(__file__)[0]


class ImproperlyConfiguredError(Exception):
    """
    Raised if the test provider is missing configuration
    """


class SRPTestCase(unittest.TestCase):
    """
    Tests for the SRPRegister class
    """
    __name__ = "SRPRegister tests"

    @classmethod
    def setUpClass(cls):
        """
        Sets up this TestCase with a simple and faked provider instance:

        * runs a threaded reactor
        * loads a mocked ProviderConfig that points to the certs in the
          leap.common.testing module.
        """
        factory = fake_provider.get_provider_factory()
        http = reactor.listenTCP(8001, factory)
        https = reactor.listenSSL(
            0, factory,
            fake_provider.OpenSSLServerContextFactory())
        get_port = lambda p: p.getHost().port
        cls.http_port = get_port(http)
        cls.https_port = get_port(https)

        provider = ProviderConfig()
        provider.get_ca_cert_path = MagicMock()
        provider.get_ca_cert_path.return_value = _get_capath()

        provider.get_api_uri = MagicMock()
        provider.get_api_uri.return_value = cls._get_https_uri()

        loaded = provider.load(path=os.path.join(
            _here, "test_provider.json"))
        if not loaded:
            raise ImproperlyConfiguredError(
                "Could not load test provider config")
        cls.register = srpregister.SRPRegister(provider_config=provider)

        cls.auth = srpauth.SRPAuth(provider)

    # helper methods

    @classmethod
    def _get_https_uri(cls):
        """
        Returns a https uri with the right https port initialized
        """
        return "https://localhost:%s" % (cls.https_port,)

    # Register tests

    def test_none_port(self):
        provider = ProviderConfig()
        provider.get_api_uri = MagicMock()
        provider.get_api_uri.return_value = "http://localhost/"
        loaded = provider.load(path=os.path.join(
            _here, "test_provider.json"))
        if not loaded:
            raise ImproperlyConfiguredError(
                "Could not load test provider config")

        register = srpregister.SRPRegister(provider_config=provider)
        self.assertEquals(register._port, "443")

    @deferred()
    def test_wrong_cert(self):
        provider = ProviderConfig()
        loaded = provider.load(path=os.path.join(
            _here, "test_provider.json"))
        provider.get_ca_cert_path = MagicMock()
        provider.get_ca_cert_path.return_value = os.path.join(
            _here,
            "wrongcert.pem")
        provider.get_api_uri = MagicMock()
        provider.get_api_uri.return_value = self._get_https_uri()
        if not loaded:
            raise ImproperlyConfiguredError(
                "Could not load test provider config")

        register = srpregister.SRPRegister(provider_config=provider)
        d = threads.deferToThread(register.register_user, "foouser_firsttime",
                                  "barpass")
        d.addCallback(self.assertFalse)
        return d

    @deferred()
    def test_register_user(self):
        """
        Checks if the registration of an unused name works as expected when
        it is the first time that we attempt to register that user, as well as
        when we request a user that is taken.
        """
        # pristine registration
        d = threads.deferToThread(self.register.register_user,
                                  "foouser_firsttime",
                                  "barpass")
        d.addCallback(self.assertTrue)
        return d

    @deferred()
    def test_second_register_user(self):
        # second registration attempt with the same user should return errors
        d = threads.deferToThread(self.register.register_user,
                                  "foouser_second",
                                  "barpass")
        d.addCallback(self.assertTrue)

        # FIXME currently we are catching this in an upper layer,
        # we could bring the error validation to the SRPRegister class
        def register_wrapper(_):
            return threads.deferToThread(self.register.register_user,
                                         "foouser_second",
                                         "barpass")
        d.addCallback(register_wrapper)
        d.addCallback(self.assertFalse)
        return d

    @deferred()
    def test_correct_http_uri(self):
        """
        Checks that registration autocorrect http uris to https ones.
        """
        HTTP_URI = "http://localhost:%s" % (self.https_port, )
        HTTPS_URI = "https://localhost:%s/1/users" % (self.https_port, )
        provider = ProviderConfig()
        provider.get_ca_cert_path = MagicMock()
        provider.get_ca_cert_path.return_value = _get_capath()
        provider.get_api_uri = MagicMock()

        # we introduce a http uri in the config file...
        provider.get_api_uri.return_value = HTTP_URI
        loaded = provider.load(path=os.path.join(
            _here, "test_provider.json"))
        if not loaded:
            raise ImproperlyConfiguredError(
                "Could not load test provider config")

        register = srpregister.SRPRegister(provider_config=provider)

        # ... and we check that we're correctly taking the HTTPS protocol
        # instead
        reg_uri = register._get_registration_uri()
        self.assertEquals(reg_uri, HTTPS_URI)
        register._get_registration_uri = MagicMock(return_value=HTTPS_URI)
        d = threads.deferToThread(register.register_user, "test_failhttp",
                                  "barpass")
        d.addCallback(self.assertTrue)

        return d



# -*- coding: utf-8 -*-
# test_streamtologger.py
# Copyright (C) 2013 LEAP
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""
tests for streamtologger
"""

try:
    import unittest2 as unittest
except ImportError:
    import unittest

import logging
import sys

from leap.bitmask.util.streamtologger import StreamToLogger
from leap.common.testing.basetest import BaseLeapTest


class SimpleLogHandler(logging.Handler):
    """
    The simplest log handler that allows to check if the log was
    delivered to the handler correctly.
    """
    def __init__(self):
        logging.Handler.__init__(self)
        self._last_log = ""
        self._last_log_level = ""

    def emit(self, record):
        self._last_log = record.getMessage()
        self._last_log_level = record.levelno

    def get_last_log(self):
        """
        Returns the last logged message by this handler.

        :return: the last logged message.
        :rtype: str
        """
        return self._last_log

    def get_last_log_level(self):
        """
        Returns the level of the last logged message by this handler.

        :return: the last logged level.
        :rtype: str
        """
        return self._last_log_level


class StreamToLoggerTest(BaseLeapTest):
    """
    StreamToLogger's tests.

    NOTE: we may need to find a way to test the use case that an exception
    is raised. I couldn't catch the output of an exception because the
    test failed if some exception is raised.
    """
    def setUp(self):
        # Create the logger
        level = logging.DEBUG
        self.logger = logging.getLogger(name='test')
        self.logger.setLevel(level)

        # Simple log handler
        self.handler = SimpleLogHandler()
        self.logger.addHandler(self.handler)

        # Preserve original values
        self._sys_stdout = sys.stdout
        self._sys_stderr = sys.stderr

        # Create the handler
        sys.stdout = StreamToLogger(self.logger, logging.DEBUG)
        sys.stderr = StreamToLogger(self.logger, logging.ERROR)

    def tearDown(self):
        # Restore original values
        sys.stdout = self._sys_stdout
        sys.stderr = self._sys_stderr

    def test_logger_starts_empty(self):
        self.assertEqual(self.handler.get_last_log(), '')

    def test_standard_output(self):
        message = 'Test string'
        print message

        log = self.handler.get_last_log()
        log_level = self.handler.get_last_log_level()

        self.assertEqual(log, message)
        self.assertEqual(log_level, logging.DEBUG)

    def test_standard_error(self):
        message = 'Test string'
        sys.stderr.write(message)

        log_level = self.handler.get_last_log_level()
        log = self.handler.get_last_log()

        self.assertEqual(log, message)
        self.assertEqual(log_level, logging.ERROR)

    def test_twisted_log(self):
        from twisted.python import log
        log.startLogging(sys.stdout)


if __name__ == "__main__":
    unittest.main(verbosity=2)



# -*- coding: utf-8 -*-
# requirement_checker.py
# Copyright (C) 2013 LEAP
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""
Utility to check the needed requirements.
"""

import os
import logging

from pkg_resources import (DistributionNotFound,
                           get_distribution,
                           Requirement,
                           resource_stream,
                           VersionConflict)

logger = logging.getLogger(__name__)


def get_requirements():
    """
    This function returns a list with requirements.
    It checks either if its running from the source or if its installed.

    :returns: a list with packages names, required for the app.
    :return type: list of str.
    """
    develop = True
    requirements = []

    try:
        # if we are running from the source
        from pkg import util
        requirements = util.parse_requirements()
    except ImportError:
        develop = False

    # if we are running from the package
    if not develop:
        requires_file_name = os.path.join(
            'leap', 'bitmask', 'util', 'reqs.txt')
        dist_name = Requirement.parse('leap.bitmask')

        try:
            with resource_stream(dist_name, requires_file_name) as stream:
                requirements = [line.strip() for line in stream]
        except Exception, e:
            logger.error("Requirements file not found. %r" % (e, ))

    return requirements


def check_requirements():
    """
    This function check the dependencies declared in the
    requirement(s) file(s) and logs the results.
    """
    logger.debug("Checking requirements...")
    requirements = get_requirements()

    for package in requirements:
        try:
            get_distribution(package)
        except VersionConflict:
            required_package = Requirement.parse(package)
            required_version = required_package.specs[0]
            required_name = required_package.key

            installed_package = get_distribution(required_name)
            installed_version = installed_package.version
            installed_location = installed_package.location

            msg = "Error: version not satisfied. "
            msg += "Expected %s, installed %s (path: %s)." % (
                required_version, installed_version, installed_location)

            result = "%s ... %s" % (package, msg)
            logger.error(result)
        except DistributionNotFound:
            msg = "Error: package not found!"
            result = "%s ... %s" % (package, msg)
            logger.error(result)
        else:
            msg = "OK"
            result = "%s ... %s" % (package, msg)
            logger.debug(result)

    logger.debug('Done')



# -*- coding: utf-8 -*-
# conductor.py
# Copyright (C) 2013 LEAP
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""
Mail Services Conductor
"""
import logging
import os

from PySide import QtCore
from zope.proxy import sameProxiedObjects

from leap.bitmask.gui import statemachines
from leap.bitmask.services.mail import imap
from leap.bitmask.services.mail import connection as mail_connection
from leap.bitmask.services.mail.smtpbootstrapper import SMTPBootstrapper
from leap.bitmask.services.mail.smtpconfig import SMTPConfig
from leap.bitmask.util import is_file

from leap.common.check import leap_assert

from leap.common.events import register as leap_register
from leap.common.events import events_pb2 as leap_events

logger = logging.getLogger(__name__)


class IMAPControl(object):
    """
    Methods related to IMAP control.
    """
    def __init__(self):
        """
        Initializes smtp variables.
        """
        self.imap_machine = None
        self.imap_service = None
        self.imap_port = None
        self.imap_factory = None
        self.imap_connection = None

        leap_register(signal=leap_events.IMAP_SERVICE_STARTED,
                      callback=self._handle_imap_events,
                      reqcbk=lambda req, resp: None)
        leap_register(signal=leap_events.IMAP_SERVICE_FAILED_TO_START,
                      callback=self._handle_imap_events,
                      reqcbk=lambda req, resp: None)

    def set_imap_connection(self, imap_connection):
        """
        Sets the imap connection to an initialized connection.

        :param imap_connection: an initialized imap connection
        :type imap_connection: IMAPConnection instance.
        """
        self.imap_connection = imap_connection

    def start_imap_service(self):
        """
        Starts imap service.
        """
        logger.debug('Starting imap service')
        leap_assert(sameProxiedObjects(self._soledad, None)
                    is not True,
                    "We need a non-null soledad for initializing imap service")
        leap_assert(sameProxiedObjects(self._keymanager, None)
                    is not True,
                    "We need a non-null keymanager for initializing imap "
                    "service")

        if self.imap_service is None:
            # first time.
            self.imap_service, \
            self.imap_port, \
            self.imap_factory = imap.start_imap_service(
                self._soledad,
                self._keymanager)
        else:
            # we have the fetcher. just start it.
            self.imap_service.start_loop()

    def stop_imap_service(self):
        """
        Stops imap service (fetcher, factory and port).
        """
        self.imap_connection.qtsigs.disconnecting_signal.emit()
        # TODO We should homogenize both services.
        if self.imap_service is not None:
            logger.debug('Stopping imap service.')
            # Stop the loop call in the fetcher
            self.imap_service.stop()
            # Stop listening on the IMAP port
            self.imap_port.stopListening()
            # Stop the protocol
            self.imap_factory.doStop()

    def fetch_incoming_mail(self):
        """
        Fetches incoming mail.
        """
        # TODO have a mutex over fetch operation.
        if self.imap_service:
            logger.debug('Client connected, fetching mail...')
            self.imap_service.fetch()

    # handle events

    def _handle_imap_events(self, req):
        """
        Callback handler for the IMAP events

        :param req: Request type
        :type req: leap.common.events.events_pb2.SignalRequest
        """
        if req.event == leap_events.IMAP_SERVICE_STARTED:
            self.on_imap_connected()
        elif req.event == leap_events.IMAP_SERVICE_FAILED_TO_START:
            self.on_imap_failed()

    # emit connection signals

    def on_imap_connecting(self):
        """
        Callback for IMAP connecting state.
        """
        self.imap_connection.qtsigs.connecting_signal.emit()

    def on_imap_connected(self):
        """
        Callback for IMAP connected state.
        """
        self.imap_connection.qtsigs.connected_signal.emit()

    def on_imap_failed(self):
        """
        Callback for IMAP failed state.
        """
        self.imap_connection.qtsigs.connetion_aborted_signal.emit()


class SMTPControl(object):

    PORT_KEY = "port"
    IP_KEY = "ip_address"

    def __init__(self):
        """
        Initializes smtp variables.
        """
        self.smtp_config = SMTPConfig()
        self.smtp_connection = None
        self.smtp_machine = None
        self._smtp_service = None
        self._smtp_port = None

        self.smtp_bootstrapper = SMTPBootstrapper()
        self.smtp_bootstrapper.download_config.connect(
            self.smtp_bootstrapped_stage)

        leap_register(signal=leap_events.SMTP_SERVICE_STARTED,
                      callback=self._handle_smtp_events,
                      reqcbk=lambda req, resp: None)
        leap_register(signal=leap_events.SMTP_SERVICE_FAILED_TO_START,
                      callback=self._handle_smtp_events,
                      reqcbk=lambda req, resp: None)

    def set_smtp_connection(self, smtp_connection):
        """
        Sets the smtp connection to an initialized connection.
        :param smtp_connection: an initialized smtp connection
        :type smtp_connection: SMTPConnection instance.
        """
        self.smtp_connection = smtp_connection

    def start_smtp_service(self, host, port, cert):
        """
        Starts the smtp service.

        :param host: the hostname of the remove SMTP server.
        :type host: str
        :param port: the port of the remote SMTP server
        :type port: str
        :param cert: the client certificate for authentication
        :type cert: str
        """
        # TODO Make the encrypted_only configurable
        # TODO pick local smtp port in a better way
        # TODO remove hard-coded port and let leap.mail set
        # the specific default.
        self.smtp_connection.qtsigs.connecting_signal.emit()
        from leap.mail.smtp import setup_smtp_relay
        self._smtp_service, self._smtp_port = setup_smtp_relay(
            port=2013,
            keymanager=self._keymanager,
            smtp_host=host,
            smtp_port=port,
            smtp_cert=cert,
            smtp_key=cert,
            encrypted_only=False)

    def stop_smtp_service(self):
        """
        Stops the smtp service (port and factory).
        """
        self.smtp_connection.qtsigs.disconnecting_signal.emit()
        # TODO We should homogenize both services.
        if self._smtp_service is not None:
            logger.debug('Stopping smtp service.')
            self._smtp_port.stopListening()
            self._smtp_service.doStop()

    @QtCore.Slot()
    def smtp_bootstrapped_stage(self, data):
        """
        SLOT
        TRIGGERS:
          self.smtp_bootstrapper.download_config

        If there was a problem, displays it, otherwise it does nothing.
        This is used for intermediate bootstrapping stages, in case
        they fail.

        :param data: result from the bootstrapping stage for Soledad
        :type data: dict
        """
        passed = data[self.smtp_bootstrapper.PASSED_KEY]
        if not passed:
            logger.error(data[self.smtp_bootstrapper.ERROR_KEY])
            return
        logger.debug("Done bootstrapping SMTP")
        self.check_smtp_config()

    def check_smtp_config(self):
        """
        Checks smtp config and tries to download smtp client cert if needed.
        Currently called when smtp_bootstrapped_stage has successfuly finished.
        """
        logger.debug("Checking SMTP config...")
        leap_assert(self.smtp_bootstrapper._provider_config,
                    "smtp bootstrapper does not have a provider_config")

        provider_config = self.smtp_bootstrapper._provider_config
        smtp_config = self.smtp_config
        hosts = smtp_config.get_hosts()
        # TODO handle more than one host and define how to choose
        if len(hosts) > 0:
            hostname = hosts.keys()[0]
            logger.debug("Using hostname %s for SMTP" % (hostname,))
            host = hosts[hostname][self.IP_KEY].encode("utf-8")
            port = hosts[hostname][self.PORT_KEY]

            client_cert = smtp_config.get_client_cert_path(
                provider_config,
                about_to_download=True)

            # XXX change this logic!
            # check_config should be called from within start_service,
            # and not the other way around.
            if not is_file(client_cert):
                self.smtp_bootstrapper._download_client_certificates()
            if os.path.isfile(client_cert):
                self.start_smtp_service(host, port, client_cert)
            else:
                logger.warning("Tried to download email client "
                               "certificate, but could not find any")

        else:
            logger.warning("No smtp hosts configured")

    # handle smtp events

    def _handle_smtp_events(self, req):
        """
        Callback handler for the SMTP events.

        :param req: Request type
        :type req: leap.common.events.events_pb2.SignalRequest
        """
        if req.event == leap_events.SMTP_SERVICE_STARTED:
            self.on_smtp_connected()
        elif req.event == leap_events.SMTP_SERVICE_FAILED_TO_START:
            self.on_smtp_failed()

    # emit connection signals

    def on_smtp_connecting(self):
        """
        Callback for SMTP connecting state.
        """
        self.smtp_connection.qtsigs.connecting_signal.emit()

    def on_smtp_connected(self):
        """
        Callback for SMTP connected state.
        """
        self.smtp_connection.qtsigs.connected_signal.emit()

    def on_smtp_failed(self):
        """
        Callback for SMTP failed state.
        """
        self.smtp_connection.qtsigs.connection_aborted_signal.emit()


class MailConductor(IMAPControl, SMTPControl):
    """
    This class encapsulates everything related to the initialization and
    process control for the mail services.
    Currently, it initializes IMAPConnection and SMPTConnection.
    """
    # XXX We could consider to use composition instead of inheritance here.

    def __init__(self, soledad, keymanager):
        """
        Initializes the mail conductor.

        :param soledad: a transparent proxy that eventually will point to a
                        Soledad Instance.
        :type soledad: zope.proxy.ProxyBase

        :param keymanager: a transparent proxy that eventually will point to a
                           Keymanager Instance.
        :type soledad: zope.proxy.ProxyBase
        """
        IMAPControl.__init__(self)
        SMTPControl.__init__(self)
        self._soledad = soledad
        self._keymanager = keymanager

        self._mail_machine = None

        self._mail_connection = mail_connection.MailConnection()

    def start_mail_machine(self, **kwargs):
        """
        Starts mail machine.
        """
        logger.debug("Starting mail state machine...")
        builder = statemachines.ConnectionMachineBuilder(self._mail_connection)
        (mail, (imap, smtp)) = builder.make_machine(**kwargs)

        # we have instantiated the connections while building the composite
        # machines, and we have to use the qtsigs instantiated there.
        # XXX we could probably use a proxy here too to make the thing
        # transparent.
        self.set_imap_connection(imap.conn)
        self.set_smtp_connection(smtp.conn)

        self._mail_machine = mail
        # XXX -------------------
        # need to keep a reference?
        #self._mail_events = mail.events
        self._mail_machine.start()

        self._imap_machine = imap
        self._imap_machine.start()
        self._smtp_machine = smtp
        self._smtp_machine.start()

    def connect_mail_signals(self, widget):
        """
        Connects the mail signals to the mail_status widget slots.

        :param widget: the widget containing the slots.
        :type widget: QtCore.QWidget
        """
        qtsigs = self._mail_connection.qtsigs
        qtsigs.connected_signal.connect(widget.mail_state_connected)
        qtsigs.connecting_signal.connect(widget.mail_state_connecting)
        qtsigs.disconnecting_signal.connect(widget.mail_state_disconnecting)
        qtsigs.disconnected_signal.connect(widget.mail_state_disconnected)



# -*- coding: utf-8 -*-
# eipbootstrapper.py
# Copyright (C) 2013 LEAP
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""
EIP bootstrapping
"""
import logging
import os

from PySide import QtCore

from leap.bitmask.config.providerconfig import ProviderConfig
from leap.bitmask.crypto.certs import download_client_cert
from leap.bitmask.services import download_service_config
from leap.bitmask.services.abstractbootstrapper import AbstractBootstrapper
from leap.bitmask.services.eip.eipconfig import EIPConfig
from leap.common import certs as leap_certs
from leap.common.check import leap_assert, leap_assert_type
from leap.common.files import check_and_fix_urw_only

logger = logging.getLogger(__name__)


class EIPBootstrapper(AbstractBootstrapper):
    """
    Sets up EIP for a provider a series of checks and emits signals
    after they are passed.
    If a check fails, the subsequent checks are not executed
    """

    # All dicts returned are of the form
    # {"passed": bool, "error": str}
    download_config = QtCore.Signal(dict)
    download_client_certificate = QtCore.Signal(dict)

    def __init__(self):
        AbstractBootstrapper.__init__(self)

        self._provider_config = None
        self._eip_config = None
        self._download_if_needed = False

    def _download_config(self, *args):
        """
        Downloads the EIP config for the given provider
        """

        leap_assert(self._provider_config,
                    "We need a provider configuration!")
        logger.debug("Downloading EIP config for %s" %
                     (self._provider_config.get_domain(),))

        self._eip_config = EIPConfig()
        download_service_config(
            self._provider_config,
            self._eip_config,
            self._session,
            self._download_if_needed)

    def _download_client_certificates(self, *args):
        """
        Downloads the EIP client certificate for the given provider
        """
        leap_assert(self._provider_config, "We need a provider configuration!")
        leap_assert(self._eip_config, "We need an eip configuration!")

        logger.debug("Downloading EIP client certificate for %s" %
                     (self._provider_config.get_domain(),))

        client_cert_path = self._eip_config.\
            get_client_cert_path(self._provider_config,
                                 about_to_download=True)

        # For re-download if something is wrong with the cert
        self._download_if_needed = self._download_if_needed and \
            not leap_certs.should_redownload(client_cert_path)

        if self._download_if_needed and \
                os.path.isfile(client_cert_path):
            check_and_fix_urw_only(client_cert_path)
            return

        download_client_cert(
            self._provider_config,
            client_cert_path,
            self._session)

    def run_eip_setup_checks(self,
                             provider_config,
                             download_if_needed=False):
        """
        Starts the checks needed for a new eip setup

        :param provider_config: Provider configuration
        :type provider_config: ProviderConfig
        """
        leap_assert(provider_config, "We need a provider config!")
        leap_assert_type(provider_config, ProviderConfig)

        self._provider_config = provider_config
        self._download_if_needed = download_if_needed

        cb_chain = [
            (self._download_config, self.download_config),
            (self._download_client_certificates,
             self.download_client_certificate)
        ]

        return self.addCallbackChain(cb_chain)



# -*- coding: utf-8 -*-
# __init__.py
# Copyright (C) 2013 LEAP
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""
leap.bitmask.services.eip module initialization
"""
import os
import tempfile

from leap.bitmask.services.eip.darwinvpnlauncher import DarwinVPNLauncher
from leap.bitmask.services.eip.linuxvpnlauncher import LinuxVPNLauncher
from leap.bitmask.services.eip.windowsvpnlauncher import WindowsVPNLauncher
from leap.bitmask.platform_init import IS_LINUX, IS_MAC, IS_WIN
from leap.common.check import leap_assert


def get_openvpn_management():
    """
    Returns the socket and port to be used for VPN

    :rtype: tuple (str, str) (host, port)
    """
    if IS_WIN:
        host = "localhost"
        port = "9876"
    else:
        # XXX cleanup this on exit too
        # XXX atexit.register ?
        host = os.path.join(tempfile.mkdtemp(prefix="leap-tmp"),
                            'openvpn.socket')
        port = "unix"

    return host, port


def get_vpn_launcher():
    """
    Return the VPN launcher for the current platform.
    """
    if not (IS_LINUX or IS_MAC or IS_WIN):
        error_msg = "VPN Launcher not implemented for this platform."
        raise NotImplementedError(error_msg)

    launcher = None
    if IS_LINUX:
        launcher = LinuxVPNLauncher
    elif IS_MAC:
        launcher = DarwinVPNLauncher
    elif IS_WIN:
        launcher = WindowsVPNLauncher

    leap_assert(launcher is not None)

    return launcher()



# -*- coding: utf-8 -*-
# test_eipconfig.py
# Copyright (C) 2013 LEAP
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""
Tests for eipconfig
"""
import copy
import json
import os
import unittest

from leap.bitmask.services.eip.eipconfig import EIPConfig
from leap.bitmask.config.providerconfig import ProviderConfig
from leap.common.testing.basetest import BaseLeapTest

from mock import Mock


sample_config = {
    "gateways": [{
        "capabilities": {
            "adblock": False,
            "filter_dns": True,
            "limited": True,
            "ports": [
                "1194",
                "443",
                "53",
                "80"],
            "protocols": [
                "tcp",
                "udp"],
            "transport": ["openvpn"],
            "user_ips": False},
        "host": "host.dev.example.org",
        "ip_address": "11.22.33.44",
        "location": "cyberspace"
    }, {
        "capabilities": {
            "adblock": False,
            "filter_dns": True,
            "limited": True,
            "ports": [
                "1194",
                "443",
                "53",
                "80"],
            "protocols": [
                "tcp",
                "udp"],
            "transport": ["openvpn"],
            "user_ips": False},
        "host": "host2.dev.example.org",
        "ip_address": "22.33.44.55",
        "location": "cyberspace"
    }
    ],
    "locations": {
        "ankara": {
            "country_code": "XX",
            "hemisphere": "S",
            "name": "Antarctica",
            "timezone": "+2"
        },
        "cyberspace": {
            "country_code": "XX",
            "hemisphere": "X",
            "name": "outer space",
            "timezone": ""
        }
    },
    "openvpn_configuration": {
        "auth": "SHA1",
        "cipher": "AES-128-CBC",
        "tls-cipher": "DHE-RSA-AES128-SHA"
    },
    "serial": 1,
    "version": 1
}


class EIPConfigTest(BaseLeapTest):

    __name__ = "eip_config_tests"

    maxDiff = None

    def setUp(self):
        self._old_ospath_exists = os.path.exists

    def tearDown(self):
        os.path.exists = self._old_ospath_exists

    def _write_config(self, data):
        """
        Helper to write some data to a temp config file.

        :param data: data to be used to save in the config file.
        :data type: dict (valid json)
        """
        self.configfile = os.path.join(self.tempdir, "eipconfig.json")
        conf = open(self.configfile, "w")
        conf.write(json.dumps(data))
        conf.close()

    def _get_eipconfig(self, fromfile=True, data=sample_config, api_ver='1'):
        """
        Helper that returns an EIPConfig object using the data parameter
        or a sample data.

        :param fromfile: sets if we should use a file or a string
        :type fromfile: bool
        :param data: sets the data to be used to load in the EIPConfig object
        :type data: dict (valid json)
        :param api_ver: the api_version schema to use.
        :type api_ver: str
        :rtype: EIPConfig
        """
        config = EIPConfig()
        config.set_api_version(api_ver)

        loaded = False
        if fromfile:
            self._write_config(data)
            loaded = config.load(self.configfile, relative=False)
        else:
            json_string = json.dumps(data)
            loaded = config.load(data=json_string)

        if not loaded:
            return None

        return config

    def test_loads_from_file(self):
        config = self._get_eipconfig()
        self.assertIsNotNone(config)

    def test_loads_from_data(self):
        config = self._get_eipconfig(fromfile=False)
        self.assertIsNotNone(config)

    def test_load_valid_config_from_file(self):
        config = self._get_eipconfig()
        self.assertIsNotNone(config)

        self.assertEqual(
            config.get_openvpn_configuration(),
            sample_config["openvpn_configuration"])

        sample_ip = sample_config["gateways"][0]["ip_address"]
        self.assertEqual(
            config.get_gateway_ip(),
            sample_ip)
        self.assertEqual(config.get_version(), sample_config["version"])
        self.assertEqual(config.get_serial(), sample_config["serial"])
        self.assertEqual(config.get_gateways(), sample_config["gateways"])
        self.assertEqual(config.get_locations(), sample_config["locations"])
        self.assertEqual(config.get_clusters(), None)

    def test_load_valid_config_from_data(self):
        config = self._get_eipconfig(fromfile=False)
        self.assertIsNotNone(config)

        self.assertEqual(
            config.get_openvpn_configuration(),
            sample_config["openvpn_configuration"])

        sample_ip = sample_config["gateways"][0]["ip_address"]
        self.assertEqual(
            config.get_gateway_ip(),
            sample_ip)

        self.assertEqual(config.get_version(), sample_config["version"])
        self.assertEqual(config.get_serial(), sample_config["serial"])
        self.assertEqual(config.get_gateways(), sample_config["gateways"])
        self.assertEqual(config.get_locations(), sample_config["locations"])
        self.assertEqual(config.get_clusters(), None)

    def test_sanitize_extra_parameters(self):
        data = copy.deepcopy(sample_config)
        data['openvpn_configuration']["extra_param"] = "FOO"
        config = self._get_eipconfig(data=data)

        self.assertEqual(
            config.get_openvpn_configuration(),
            sample_config["openvpn_configuration"])

    def test_sanitize_non_allowed_chars(self):
        data = copy.deepcopy(sample_config)
        data['openvpn_configuration']["auth"] = "SHA1;"
        config = self._get_eipconfig(data=data)

        self.assertEqual(
            config.get_openvpn_configuration(),
            sample_config["openvpn_configuration"])

        data = copy.deepcopy(sample_config)
        data['openvpn_configuration']["auth"] = "SHA1>`&|"
        config = self._get_eipconfig(data=data)

        self.assertEqual(
            config.get_openvpn_configuration(),
            sample_config["openvpn_configuration"])

    def test_sanitize_lowercase(self):
        data = copy.deepcopy(sample_config)
        data['openvpn_configuration']["auth"] = "shaSHA1"
        config = self._get_eipconfig(data=data)

        self.assertEqual(
            config.get_openvpn_configuration(),
            sample_config["openvpn_configuration"])

    def test_all_characters_invalid(self):
        data = copy.deepcopy(sample_config)
        data['openvpn_configuration']["auth"] = "sha&*!@#;"
        config = self._get_eipconfig(data=data)

        self.assertEqual(
            config.get_openvpn_configuration(),
            {'cipher': 'AES-128-CBC',
             'tls-cipher': 'DHE-RSA-AES128-SHA'})

    def test_sanitize_bad_ip(self):
        data = copy.deepcopy(sample_config)
        data['gateways'][0]["ip_address"] = "11.22.33.44;"
        config = self._get_eipconfig(data=data)

        self.assertEqual(config.get_gateway_ip(), None)

        data = copy.deepcopy(sample_config)
        data['gateways'][0]["ip_address"] = "11.22.33.44`"
        config = self._get_eipconfig(data=data)

        self.assertEqual(config.get_gateway_ip(), None)

    def test_default_gateway_on_unknown_index(self):
        config = self._get_eipconfig()
        sample_ip = sample_config["gateways"][0]["ip_address"]
        self.assertEqual(config.get_gateway_ip(999), sample_ip)

    def test_get_gateway_by_index(self):
        config = self._get_eipconfig()
        sample_ip_0 = sample_config["gateways"][0]["ip_address"]
        sample_ip_1 = sample_config["gateways"][1]["ip_address"]
        self.assertEqual(config.get_gateway_ip(0), sample_ip_0)
        self.assertEqual(config.get_gateway_ip(1), sample_ip_1)

    def test_get_client_cert_path_as_expected(self):
        config = self._get_eipconfig()
        provider_config = ProviderConfig()

        # mock 'get_domain' so we don't need to load a config
        provider_domain = 'test.provider.com'
        provider_config.get_domain = Mock(return_value=provider_domain)

        expected_path = os.path.join('leap', 'providers',
                                     provider_domain, 'keys', 'client',
                                     'openvpn.pem')

        # mock 'os.path.exists' so we don't get an error for unexisting file
        os.path.exists = Mock(return_value=True)
        cert_path = config.get_client_cert_path(provider_config)

        self.assertTrue(cert_path.endswith(expected_path))

    def test_get_client_cert_path_about_to_download(self):
        config = self._get_eipconfig()
        provider_config = ProviderConfig()

        # mock 'get_domain' so we don't need to load a config
        provider_domain = 'test.provider.com'
        provider_config.get_domain = Mock(return_value=provider_domain)

        expected_path = os.path.join('leap', 'providers',
                                     provider_domain, 'keys', 'client',
                                     'openvpn.pem')

        cert_path = config.get_client_cert_path(
            provider_config, about_to_download=True)

        self.assertTrue(cert_path.endswith(expected_path))

    def test_get_client_cert_path_fails(self):
        config = self._get_eipconfig()
        provider_config = ProviderConfig()

        # mock 'get_domain' so we don't need to load a config
        provider_domain = 'test.provider.com'
        provider_config.get_domain = Mock(return_value=provider_domain)

        with self.assertRaises(AssertionError):
            config.get_client_cert_path(provider_config)

    def test_fails_without_api_set(self):
        config = EIPConfig()
        with self.assertRaises(AssertionError):
            config.load('non-relevant-path')

    def test_fails_with_api_without_schema(self):
        with self.assertRaises(AssertionError):
            self._get_eipconfig(api_ver='123')

if __name__ == "__main__":
    unittest.main()



# -*- coding: utf-8 -*-
# twisted.py
# Copyright (C) 2013 LEAP
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""
Twisted services launched by the client
"""
import logging

from twisted.application.service import Application
#from twisted.internet.task import LoopingCall

logger = logging.getLogger(__name__)


def task():
    """
    stub periodic task, mainly for tests.
    DELETE-ME when there's real meat here :)
    """
    from datetime import datetime
    logger.debug("hi there %s", datetime.now())


def leap_services():
    """
    Check which twisted services are enabled and
    register them.
    """
    logger.debug('starting leap services')
    application = Application("Bitmask Local Services")
    #lc = LoopingCall(task)
    #lc.start(5)
    return application



# -*- coding: utf-8 -*-
# soledadspec.py
# Copyright (C) 2013 LEAP
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

# Schemas dict
# To add a schema for a version you should follow the form:
# { '1': schema_v1, '2': schema_v2, ... etc }
# so for instance, to add the '2' version, you should do:
# soledad_config_spec['2'] = schema_v2
soledad_config_spec = {}

soledad_config_spec['1'] = {
    'description': 'sample soledad service config',
    'type': 'object',
    'properties': {
        'serial': {
            'type': int,
            'default': 1,
            'required': ["True"]
        },
        'version': {
            'type': int,
            'default': 1,
            'required': ["True"]
        },
        'hosts': {
            'type': dict,
            'default': {
                "python": {
                    "hostname": "someprovider",
                    "ip_address": "1.1.1.1",
                    "location": "loc",
                    "port": 1111
                },
            },
        },
        'locations': {
            'type': dict,
            'default': {
                "locations": {
                    "ankara": {
                        "country_code": "TR",
                        "hemisphere": "N",
                        "name": "loc",
                        "timezone": "+0"
                    }
                }
            }
        }
    }
}


def get_schema(version):
    """
    Returns the schema corresponding to the version given.

    :param version: the version of the schema to get.
    :type version: str
    :rtype: dict or None if the version is not supported.
    """
    schema = soledad_config_spec.get(version, None)
    return schema



# -*- coding: utf-8 -*-
# vpnlauncher.py
# Copyright (C) 2013 LEAP
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""
Platform independant VPN launcher interface.
"""
import getpass
import logging
import os
import stat

from abc import ABCMeta, abstractmethod
from functools import partial

from leap.bitmask.config import flags
from leap.bitmask.config.leapsettings import LeapSettings
from leap.bitmask.config.providerconfig import ProviderConfig
from leap.bitmask.services.eip.eipconfig import EIPConfig, VPNGatewaySelector
from leap.bitmask.util import first
from leap.bitmask.util import get_path_prefix
from leap.common.check import leap_assert, leap_assert_type
from leap.common.files import which

logger = logging.getLogger(__name__)


class VPNLauncherException(Exception):
    pass


class OpenVPNNotFoundException(VPNLauncherException):
    pass


def _has_updown_scripts(path, warn=True):
    """
    Checks the existence of the up/down scripts and its
    exec bit if applicable.

    :param path: the path to be checked
    :type path: str

    :param warn: whether we should log the absence
    :type warn: bool

    :rtype: bool
    """
    is_file = os.path.isfile(path)
    if warn and not is_file:
        logger.error("Could not find up/down script %s. "
                     "Might produce DNS leaks." % (path,))

    # XXX check if applies in win
    is_exe = False
    try:
        is_exe = (stat.S_IXUSR & os.stat(path)[stat.ST_MODE] != 0)
    except OSError as e:
        logger.warn("%s" % (e,))
    if warn and not is_exe:
        logger.error("Up/down script %s is not executable. "
                     "Might produce DNS leaks." % (path,))
    return is_file and is_exe


def _has_other_files(path, warn=True):
    """
    Checks the existence of other important files.

    :param path: the path to be checked
    :type path: str

    :param warn: whether we should log the absence
    :type warn: bool

    :rtype: bool
    """
    is_file = os.path.isfile(path)
    if warn and not is_file:
        logger.warning("Could not find file during checks: %s. " % (
            path,))
    return is_file


class VPNLauncher(object):
    """
    Abstract launcher class
    """
    __metaclass__ = ABCMeta

    UPDOWN_FILES = None
    OTHER_FILES = None
    UP_SCRIPT = None
    DOWN_SCRIPT = None

    @classmethod
    @abstractmethod
    def get_vpn_command(kls, eipconfig, providerconfig,
                        socket_host, socket_port, openvpn_verb=1):
        """
        Returns the platform dependant vpn launching command

        Might raise:
            OpenVPNNotFoundException,
            VPNLauncherException.

        :param eipconfig: eip configuration object
        :type eipconfig: EIPConfig
        :param providerconfig: provider specific configuration
        :type providerconfig: ProviderConfig
        :param socket_host: either socket path (unix) or socket IP
        :type socket_host: str
        :param socket_port: either string "unix" if it's a unix socket,
                            or port otherwise
        :type socket_port: str
        :param openvpn_verb: the openvpn verbosity wanted
        :type openvpn_verb: int

        :return: A VPN command ready to be launched.
        :rtype: list
        """
        leap_assert_type(eipconfig, EIPConfig)
        leap_assert_type(providerconfig, ProviderConfig)

        kwargs = {}
        if flags.STANDALONE:
            kwargs['path_extension'] = os.path.join(
                get_path_prefix(), "..", "apps", "eip")

        openvpn_possibilities = which(kls.OPENVPN_BIN, **kwargs)
        if len(openvpn_possibilities) == 0:
            raise OpenVPNNotFoundException()

        openvpn = first(openvpn_possibilities)
        args = []

        args += [
            '--setenv', "LEAPOPENVPN", "1"
        ]

        if openvpn_verb is not None:
            args += ['--verb', '%d' % (openvpn_verb,)]

        gateways = []
        leap_settings = LeapSettings()
        domain = providerconfig.get_domain()
        gateway_conf = leap_settings.get_selected_gateway(domain)

        if gateway_conf == leap_settings.GATEWAY_AUTOMATIC:
            gateway_selector = VPNGatewaySelector(eipconfig)
            gateways = gateway_selector.get_gateways()
        else:
            gateways = [gateway_conf]

        if not gateways:
            logger.error('No gateway was found!')
            raise VPNLauncherException('No gateway was found!')

        logger.debug("Using gateways ips: {0}".format(', '.join(gateways)))

        for gw in gateways:
            args += ['--remote', gw, '1194', 'udp']

        args += [
            '--client',
            '--dev', 'tun',
            ##############################################################
            # persist-tun makes ping-restart fail because it leaves a
            # broken routing table
            ##############################################################
            # '--persist-tun',
            '--persist-key',
            '--tls-client',
            '--remote-cert-tls',
            'server'
        ]

        openvpn_configuration = eipconfig.get_openvpn_configuration()
        for key, value in openvpn_configuration.items():
            args += ['--%s' % (key,), value]

        user = getpass.getuser()

        ##############################################################
        # The down-root plugin fails in some situations, so we don't
        # drop privs for the time being
        ##############################################################
        # args += [
        #     '--user', user,
        #     '--group', grp.getgrgid(os.getgroups()[-1]).gr_name
        # ]

        if socket_port == "unix":  # that's always the case for linux
            args += [
                '--management-client-user', user
            ]

        args += [
            '--management-signal',
            '--management', socket_host, socket_port,
            '--script-security', '2'
        ]

        if kls.UP_SCRIPT is not None:
            if _has_updown_scripts(kls.UP_SCRIPT):
                args += [
                    '--up', '\"%s\"' % (kls.UP_SCRIPT,),
                ]

        if kls.DOWN_SCRIPT is not None:
            if _has_updown_scripts(kls.DOWN_SCRIPT):
                args += [
                    '--down', '\"%s\"' % (kls.DOWN_SCRIPT,)
                ]

        ###########################################################
        # For the time being we are disabling the usage of the
        # down-root plugin, because it doesn't quite work as
        # expected (i.e. it doesn't run route -del as root
        # when finishing, so it fails to properly
        # restart/quit)
        ###########################################################
        # if _has_updown_scripts(kls.OPENVPN_DOWN_PLUGIN):
        #     args += [
        #         '--plugin', kls.OPENVPN_DOWN_ROOT,
        #         '\'%s\'' % kls.DOWN_SCRIPT  # for OSX
        #         '\'script_type=down %s\'' % kls.DOWN_SCRIPT  # for Linux
        #     ]

        args += [
            '--cert', eipconfig.get_client_cert_path(providerconfig),
            '--key', eipconfig.get_client_cert_path(providerconfig),
            '--ca', providerconfig.get_ca_cert_path()
        ]

        args += [
            '--ping', '10',
            '--ping-restart', '30']

        command_and_args = [openvpn] + args
        return command_and_args

    @classmethod
    def get_vpn_env(kls):
        """
        Returns a dictionary with the custom env for the platform.
        This is mainly used for setting LD_LIBRARY_PATH to the correct
        path when distributing a standalone client

        :rtype: dict
        """
        return {}

    @classmethod
    def missing_updown_scripts(kls):
        """
        Returns what updown scripts are missing.

        :rtype: list
        """
        leap_assert(kls.UPDOWN_FILES is not None,
                    "Need to define UPDOWN_FILES for this particular "
                    "launcher before calling this method")
        file_exist = partial(_has_updown_scripts, warn=False)
        zipped = zip(kls.UPDOWN_FILES, map(file_exist, kls.UPDOWN_FILES))
        missing = filter(lambda (path, exists): exists is False, zipped)
        return [path for path, exists in missing]

    @classmethod
    def missing_other_files(kls):
        """
        Returns what other important files are missing during startup.
        Same as missing_updown_scripts but does not check for exec bit.

        :rtype: list
        """
        leap_assert(kls.OTHER_FILES is not None,
                    "Need to define OTHER_FILES for this particular "
                    "auncher before calling this method")
        file_exist = partial(_has_other_files, warn=False)
        zipped = zip(kls.OTHER_FILES, map(file_exist, kls.OTHER_FILES))
        missing = filter(lambda (path, exists): exists is False, zipped)
        return [path for path, exists in missing]



# -*- coding: utf-8 -*-
# imap.py
# Copyright (C) 2013 LEAP
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""
Initialization of imap service
"""
import logging
import os
#import sys

from leap.mail.imap.service import imap
#from twisted.python import log

logger = logging.getLogger(__name__)

# The name of the environment variable that has to be
# set to override the default time value, in seconds.
INCOMING_CHECK_PERIOD_ENV = "BITMASK_MAILCHECK_PERIOD"


def get_mail_check_period():
    """
    Tries to get the value of the environment variable for
    overriding the period for incoming mail fetch.
    """
    period = None
    period_str = os.environ.get(INCOMING_CHECK_PERIOD_ENV, None)
    try:
        period = int(period_str)
    except (ValueError, TypeError):
        if period is not None:
            logger.warning("BAD value found for %s: %s" % (
                INCOMING_CHECK_PERIOD_ENV,
                period_str))
    except Exception as exc:
        logger.warning("Unhandled error while getting %s: %r" % (
            INCOMING_CHECK_PERIOD_ENV,
            exc))
    return period


def start_imap_service(*args, **kwargs):
    """
    Initializes and run imap service.

    :returns: twisted.internet.task.LoopingCall instance
    """
    logger.debug('Launching imap service')

    override_period = get_mail_check_period()
    if override_period:
        kwargs['check_period'] = override_period

    # Uncomment the next two lines to get a separate debugging log
    # TODO handle this by a separate flag.
    #log.startLogging(open('/tmp/leap-imap.log', 'w'))
    #log.startLogging(sys.stdout)

    return imap.run_service(*args, **kwargs)



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'src/leap/bitmask/gui/ui/mail_status.ui'
#
# Created: Wed Oct 30 17:39:24 2013
#      by: pyside-uic 0.2.15 running on PySide 1.2.1
#
# WARNING! All changes made in this file will be lost!

from PySide import QtCore, QtGui

class Ui_MailStatusWidget(object):
    def setupUi(self, MailStatusWidget):
        MailStatusWidget.setObjectName("MailStatusWidget")
        MailStatusWidget.resize(400, 79)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Preferred, QtGui.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(MailStatusWidget.sizePolicy().hasHeightForWidth())
        MailStatusWidget.setSizePolicy(sizePolicy)
        self.gridLayout = QtGui.QGridLayout(MailStatusWidget)
        self.gridLayout.setContentsMargins(-1, 0, -1, -1)
        self.gridLayout.setObjectName("gridLayout")
        self.gridLayout_3 = QtGui.QGridLayout()
        self.gridLayout_3.setObjectName("gridLayout_3")
        spacerItem = QtGui.QSpacerItem(40, 20, QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Minimum)
        self.gridLayout_3.addItem(spacerItem, 1, 1, 1, 1)
        self.lblMailStatus = QtGui.QLabel(MailStatusWidget)
        self.lblMailStatus.setStyleSheet("color: rgb(80, 80, 80);")
        self.lblMailStatus.setObjectName("lblMailStatus")
        self.gridLayout_3.addWidget(self.lblMailStatus, 2, 0, 1, 3)
        self.lblMailStatusIcon = QtGui.QLabel(MailStatusWidget)
        self.lblMailStatusIcon.setMaximumSize(QtCore.QSize(24, 24))
        self.lblMailStatusIcon.setText("")
        self.lblMailStatusIcon.setPixmap(QtGui.QPixmap(":/images/black/32/off.png"))
        self.lblMailStatusIcon.setScaledContents(True)
        self.lblMailStatusIcon.setObjectName("lblMailStatusIcon")
        self.gridLayout_3.addWidget(self.lblMailStatusIcon, 1, 3, 1, 1)
        self.label_4 = QtGui.QLabel(MailStatusWidget)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Preferred, QtGui.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.label_4.sizePolicy().hasHeightForWidth())
        self.label_4.setSizePolicy(sizePolicy)
        self.label_4.setObjectName("label_4")
        self.gridLayout_3.addWidget(self.label_4, 1, 0, 1, 1)
        spacerItem1 = QtGui.QSpacerItem(0, 5, QtGui.QSizePolicy.Minimum, QtGui.QSizePolicy.Fixed)
        self.gridLayout_3.addItem(spacerItem1, 0, 0, 1, 1)
        self.gridLayout.addLayout(self.gridLayout_3, 0, 1, 2, 1)
        self.label = QtGui.QLabel(MailStatusWidget)
        self.label.setText("")
        self.label.setPixmap(QtGui.QPixmap(":/images/black/32/email.png"))
        self.label.setObjectName("label")
        self.gridLayout.addWidget(self.label, 0, 0, 1, 1)

        self.retranslateUi(MailStatusWidget)
        QtCore.QMetaObject.connectSlotsByName(MailStatusWidget)

    def retranslateUi(self, MailStatusWidget):
        MailStatusWidget.setWindowTitle(QtGui.QApplication.translate("MailStatusWidget", "Form", None, QtGui.QApplication.UnicodeUTF8))
        self.lblMailStatus.setText(QtGui.QApplication.translate("MailStatusWidget", "You must login to use encrypted email.", None, QtGui.QApplication.UnicodeUTF8))
        self.label_4.setText(QtGui.QApplication.translate("MailStatusWidget", "Email", None, QtGui.QApplication.UnicodeUTF8))

import mainwindow_rc



# -*- coding: utf-8 -*-
# abstractbootstrapper.py
# Copyright (C) 2013 LEAP
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""
Abstract bootstrapper implementation
"""
import logging

import requests

from functools import partial

from PySide import QtCore
from twisted.internet import threads

from leap.common.check import leap_assert, leap_assert_type

logger = logging.getLogger(__name__)


class AbstractBootstrapper(QtCore.QObject):
    """
    Abstract Bootstrapper that implements the needed deferred callbacks
    """

    PASSED_KEY = "passed"
    ERROR_KEY = "error"

    def __init__(self, bypass_checks=False):
        """
        Constructor for the abstract bootstrapper

        :param bypass_checks: Set to true if the app should bypass
                              first round of checks for CA
                              certificates at bootstrap
        :type bypass_checks: bool
        """
        QtCore.QObject.__init__(self)

        leap_assert(self._gui_errback.im_func ==
                    AbstractBootstrapper._gui_errback.im_func,
                    "Cannot redefine _gui_errback")
        leap_assert(self._errback.im_func ==
                    AbstractBootstrapper._errback.im_func,
                    "Cannot redefine _errback")
        leap_assert(self._gui_notify.im_func ==
                    AbstractBootstrapper._gui_notify.im_func,
                    "Cannot redefine _gui_notify")

        # **************************************************** #
        # Dependency injection helpers, override this for more
        # granular testing
        self._fetcher = requests
        # **************************************************** #

        self._session = self._fetcher.session()
        self._bypass_checks = bypass_checks
        self._signal_to_emit = None
        self._err_msg = None

    def _gui_errback(self, failure):
        """
        Errback used to notify the GUI of a problem, it should be used
        as the last errback of the whole chain.

        Traps all exceptions if a signal is defined, otherwise it just
        lets it continue.

        NOTE: This method is final, it should not be redefined.

        :param failure: failure object that Twisted generates
        :type failure: twisted.python.failure.Failure
        """
        if self._signal_to_emit:
            err_msg = self._err_msg \
                if self._err_msg is not None \
                else str(failure.value)
            self._signal_to_emit.emit({
                self.PASSED_KEY: False,
                self.ERROR_KEY: err_msg
            })
            failure.trap(Exception)

    def _errback(self, failure, signal=None):
        """
        Regular errback used for the middle of the chain. If it's
        executed, the first one will set the signal to emit as
        failure.

        NOTE: This method is final, it should not be redefined.

        :param failure: failure object that Twisted generates
        :type failure: twisted.python.failure.Failure
        :param signal: Signal to emit if it fails here first
        :type signal: QtCore.SignalInstance

        :returns: failure object that Twisted generates
        :rtype: twisted.python.failure.Failure
        """
        if self._signal_to_emit is None:
            self._signal_to_emit = signal
        return failure

    def _gui_notify(self, _, signal=None):
        """
        Callback used to notify the GUI of a success. Will emit signal
        if specified

        NOTE: This method is final, it should not be redefined.

        :param _: IGNORED. Returned from the previous callback
        :type _: IGNORED
        :param signal: Signal to emit if it fails here first
        :type signal: QtCore.SignalInstance
        """
        if signal:
            signal.emit({self.PASSED_KEY: True, self.ERROR_KEY: ""})

    def _callback_threader(self, cb, res, *args, **kwargs):
        return threads.deferToThread(cb, res, *args, **kwargs)

    def addCallbackChain(self, callbacks):
        """
        Creates a callback/errback chain on another thread using
        deferToThread and adds the _gui_errback to the end to notify
        the GUI on an error.

        :param callbacks: List of tuples of callbacks and the signal
                          associated to that callback
        :type callbacks: list(tuple(func, func))

        :returns: the defer with the callback chain
        :rtype: deferred
        """
        leap_assert_type(callbacks, list)

        self._signal_to_emit = None
        self._err_msg = None

        d = None
        for cb, sig in callbacks:
            if d is None:
                d = threads.deferToThread(cb)
            else:
                d.addCallback(partial(self._callback_threader, cb))
            d.addErrback(self._errback, signal=sig)
            d.addCallback(self._gui_notify, signal=sig)
        d.addErrback(self._gui_errback)
        return d



# -*- coding: utf-8 -*-
# supportedapis.py
# Copyright (C) 2013 LEAP
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""
API Support check.
"""


class SupportedAPIs(object):
    """
    Class responsible of checking for API compatibility.
    """
    SUPPORTED_APIS = ["1"]

    @classmethod
    def supports(self, api_version):
        """
        :param api_version: the version number of the api that we need to check
        :type api_version: str

        :returns: if that version is supported or not.
        :return type: bool
        """
        return api_version in self.SUPPORTED_APIS



# -*- coding: utf-8 -*-
# leapsettings.py
# Copyright (C) 2013 LEAP
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""
QSettings abstraction
"""
import os
import logging

from PySide import QtCore

from leap.common.check import leap_assert, leap_assert_type
from leap.bitmask.util import get_path_prefix

logger = logging.getLogger(__name__)


def to_bool(val):
    """
    Returns the boolean value corresponding to val. Will return False
    in case val is not a string or something that behaves like one.

    :param val: value to cast
    :type val: either bool already or str

    :rtype: bool
    """
    if isinstance(val, bool):
        return val

    bool_val = False
    try:
        bool_val = val.lower() == "true"
    except:
        pass

    return bool_val


class LeapSettings(object):
    """
    Leap client QSettings wrapper
    """

    CONFIG_NAME = "leap.conf"

    # keys
    GEOMETRY_KEY = "Geometry"
    WINDOWSTATE_KEY = "WindowState"
    USER_KEY = "User"
    PROVIDER_KEY = "Provider"
    REMEMBER_KEY = "RememberUserAndPass"
    DEFAULTPROVIDER_KEY = "DefaultProvider"
    AUTOSTARTEIP_KEY = "AutoStartEIP"
    ALERTMISSING_KEY = "AlertMissingScripts"
    GATEWAY_KEY = "Gateway"
    PINNED_KEY = "Pinned"
    SKIPFIRSTRUN_KEY = "SkipFirstRun"

    # values
    GATEWAY_AUTOMATIC = "Automatic"

    def __init__(self):
        settings_path = os.path.join(get_path_prefix(),
                                     "leap", self.CONFIG_NAME)

        self._settings = QtCore.QSettings(settings_path,
                                          QtCore.QSettings.IniFormat)

    def get_geometry(self):
        """
        Returns the saved geometry or None if it wasn't saved

        :rtype: bytearray or None
        """
        return self._settings.value(self.GEOMETRY_KEY, None)

    def set_geometry(self, geometry):
        """
        Saves the geometry to the settings

        :param geometry: bytearray representing the geometry
        :type geometry: bytearray
        """
        leap_assert(geometry, "We need a geometry")
        self._settings.setValue(self.GEOMETRY_KEY, geometry)

    def get_windowstate(self):
        """
        Returns the window state or None if it wasn't saved

        :rtype: bytearray or None
        """
        return self._settings.value(self.WINDOWSTATE_KEY, None)

    def set_windowstate(self, windowstate):
        """
        Saves the window state to the settings

        :param windowstate: bytearray representing the window state
        :type windowstate: bytearray
        """
        leap_assert(windowstate, "We need a window state")
        self._settings.setValue(self.WINDOWSTATE_KEY, windowstate)

    def get_configured_providers(self):
        """
        Returns the configured providers based on the file structure in the
        settings directory.

        :rtype: list of str
        """
        # TODO: check which providers have a valid certificate among
        # other things, not just the directories
        providers = []
        try:
            providers_path = os.path.join(get_path_prefix(),
                                          "leap", "providers")
            providers = os.listdir(providers_path)
        except Exception as e:
            logger.debug("Error listing providers, assume there are none. %r"
                         % (e,))

        return providers

    def is_pinned_provider(self, domain):
        """
        Returns True if the domain 'domain' is pinned with the application.
                False otherwise.

        :param provider: provider domain
        :type provider: str

        :rtype: bool
        """
        leap_assert(len(domain) > 0, "We need a nonempty domain.")
        pinned_key = "{0}/{1}".format(domain, self.PINNED_KEY)
        result = to_bool(self._settings.value(pinned_key, False))

        return result

    def get_selected_gateway(self, provider):
        """
        Returns the configured gateway for the given provider.

        :param provider: provider domain
        :type provider: str

        :rtype: str
        """
        leap_assert(len(provider) > 0, "We need a nonempty provider")
        gateway_key = "{0}/{1}".format(provider, self.GATEWAY_KEY)
        gateway = self._settings.value(gateway_key, self.GATEWAY_AUTOMATIC)

        return gateway

    def set_selected_gateway(self, provider, gateway):
        """
        Saves the configured gateway for the given provider

        :param provider: provider domain
        :type provider: str

        :param gateway: gateway to use as default
        :type gateway: str
        """

        leap_assert(len(provider) > 0, "We need a nonempty provider")
        leap_assert_type(gateway, (str, unicode))

        gateway_key = "{0}/{1}".format(provider, self.GATEWAY_KEY)
        self._settings.setValue(gateway_key, gateway)

    def get_enabled_services(self, provider):
        """
        Returns a list of enabled services for the given provider

        :param provider: provider domain
        :type provider: str

        :rtype: list of str
        """

        leap_assert(len(provider) > 0, "We need a nonempty provider")
        enabled_services = self._settings.value("%s/Services" % (provider,),
                                                [])
        if isinstance(enabled_services, (str, unicode)):
            enabled_services = enabled_services.split(",")

        return enabled_services

    def set_enabled_services(self, provider, services):
        """
        Saves the list of enabled services for the given provider

        :param provider: provider domain
        :type provider: str

        :param services: list of services to save
        :type services: list of str
        """

        leap_assert(len(provider) > 0, "We need a nonempty provider")
        leap_assert_type(services, list)

        key = "{0}/Services".format(provider)
        if not services:
            # if there are no enabled services we don't need that key
            self._settings.remove(key)
        else:
            self._settings.setValue(key, services)

    def get_user(self):
        """
        Returns the configured user to remember, None if there isn't one

        :rtype: str or None
        """
        return self._settings.value(self.USER_KEY, None)

    def set_user(self, user):
        """
        Saves the user to remember

        :param user: user name to remember
        :type user: str
        """
        leap_assert(len(user) > 0, "We cannot save an empty user")
        self._settings.setValue(self.USER_KEY, user)

    def get_provider(self):
        """
        Returns the configured provider to remember, None if there isn't one

        :rtype: str or None
        """
        return self._settings.value(self.PROVIDER_KEY, None)

    def set_provider(self, provider):
        """
        Saves the provider to remember

        :param provider: provider name to remember
        :type provider: str
        """
        leap_assert(len(provider) > 0, "We cannot save an empty provider")
        self._settings.setValue(self.PROVIDER_KEY, provider)

    def get_remember(self):
        """
        Returns the value of the remember selection.

        :rtype: bool
        """
        return to_bool(self._settings.value(self.REMEMBER_KEY, False))

    def set_remember(self, remember):
        """
        Sets wheter the app should remember username and password

        :param remember: True if the app should remember username and
            password, False otherwise
        :rtype: bool
        """
        leap_assert_type(remember, bool)
        self._settings.setValue(self.REMEMBER_KEY, remember)

    def get_defaultprovider(self):
        """
        Returns the default provider to be used for autostarting EIP

        :rtype: str or None
        """
        return self._settings.value(self.DEFAULTPROVIDER_KEY, None)

    def set_defaultprovider(self, provider):
        """
        Sets the default provider to be used for autostarting EIP

        :param provider: provider to use
        :type provider: str or None
        """
        if provider is None:
            self._settings.remove(self.DEFAULTPROVIDER_KEY)
        else:
            self._settings.setValue(self.DEFAULTPROVIDER_KEY, provider)

    def get_autostart_eip(self):
        """
        Gets whether the app should autostart EIP.

        :rtype: bool
        """
        return to_bool(self._settings.value(self.AUTOSTARTEIP_KEY, False))

    def set_autostart_eip(self, autostart):
        """
        Sets whether the app should autostart EIP.

        :param autostart: True if we should try to autostart EIP.
        :type autostart: bool
        """
        leap_assert_type(autostart, bool)
        self._settings.setValue(self.AUTOSTARTEIP_KEY, autostart)

    def get_alert_missing_scripts(self):
        """
        Returns the setting for alerting of missing up/down scripts.

        :rtype: bool
        """
        return to_bool(self._settings.value(self.ALERTMISSING_KEY, True))

    def set_alert_missing_scripts(self, value):
        """
        Sets the setting for alerting of missing up/down scripts.

        :param value: the value to set
        :type value: bool
        """
        leap_assert_type(value, bool)
        self._settings.setValue(self.ALERTMISSING_KEY, value)

    def get_skip_first_run(self):
        """
        Gets the setting for skip running the first run wizard.

        :returns: if the first run wizard should be skipped or not
        :rtype: bool
        """
        return to_bool(self._settings.value(self.SKIPFIRSTRUN_KEY, False))

    def set_skip_first_run(self, skip):
        """
        Gets the setting for skip the first run wizard.

        :param skip: if the first run wizard should be skipped or not
        :type skip: bool
        """
        leap_assert_type(skip, bool)
        self._settings.setValue(self.SKIPFIRSTRUN_KEY, skip)



# -*- coding: utf-8 -*-
# __init.py
# Copyright (C) 2013 LEAP
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""
Module initialization for leap.bitmask.provider
"""
import os
from leap.common.check import leap_assert


def get_provider_path(domain):
    """
    Returns relative path for provider config.

    :param domain: the domain to which this providerconfig belongs to.
    :type domain: str
    :returns: the path
    :rtype: str
    """
    leap_assert(domain is not None, "get_provider_path: We need a domain")
    return os.path.join("leap", "providers", domain, "provider.json")



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'src/leap/bitmask/gui/ui/mainwindow.ui'
#
# Created: Wed Oct 30 17:39:23 2013
#      by: pyside-uic 0.2.15 running on PySide 1.2.1
#
# WARNING! All changes made in this file will be lost!

from PySide import QtCore, QtGui

class Ui_MainWindow(object):
    def setupUi(self, MainWindow):
        MainWindow.setObjectName("MainWindow")
        MainWindow.resize(524, 722)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Fixed, QtGui.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(MainWindow.sizePolicy().hasHeightForWidth())
        MainWindow.setSizePolicy(sizePolicy)
        MainWindow.setMinimumSize(QtCore.QSize(524, 0))
        MainWindow.setMaximumSize(QtCore.QSize(524, 16777215))
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/images/mask-icon.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        MainWindow.setWindowIcon(icon)
        MainWindow.setInputMethodHints(QtCore.Qt.ImhHiddenText)
        MainWindow.setIconSize(QtCore.QSize(128, 128))
        self.centralwidget = QtGui.QWidget(MainWindow)
        self.centralwidget.setObjectName("centralwidget")
        self.gridLayout = QtGui.QGridLayout(self.centralwidget)
        self.gridLayout.setContentsMargins(0, 0, 0, 0)
        self.gridLayout.setSpacing(0)
        self.gridLayout.setObjectName("gridLayout")
        self.scrollArea = QtGui.QScrollArea(self.centralwidget)
        self.scrollArea.setFrameShape(QtGui.QFrame.NoFrame)
        self.scrollArea.setFrameShadow(QtGui.QFrame.Plain)
        self.scrollArea.setLineWidth(0)
        self.scrollArea.setHorizontalScrollBarPolicy(QtCore.Qt.ScrollBarAlwaysOff)
        self.scrollArea.setWidgetResizable(True)
        self.scrollArea.setObjectName("scrollArea")
        self.scrollAreaWidgetContents = QtGui.QWidget()
        self.scrollAreaWidgetContents.setGeometry(QtCore.QRect(0, 0, 524, 636))
        self.scrollAreaWidgetContents.setObjectName("scrollAreaWidgetContents")
        self.verticalLayout = QtGui.QVBoxLayout(self.scrollAreaWidgetContents)
        self.verticalLayout.setSpacing(0)
        self.verticalLayout.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout.setObjectName("verticalLayout")
        self.eipWidget = QtGui.QWidget(self.scrollAreaWidgetContents)
        self.eipWidget.setObjectName("eipWidget")
        self.verticalLayout_2 = QtGui.QVBoxLayout(self.eipWidget)
        self.verticalLayout_2.setSpacing(0)
        self.verticalLayout_2.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout_2.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.frame_2 = QtGui.QFrame(self.eipWidget)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Preferred, QtGui.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.frame_2.sizePolicy().hasHeightForWidth())
        self.frame_2.setSizePolicy(sizePolicy)
        self.frame_2.setStyleSheet("QFrame{background-color: qlineargradient(spread:pad, x1:0, y1:1, x2:0, y2:0, stop:0 rgba(160, 160, 160, 128), stop:1 rgba(255, 255, 255, 0));}")
        self.frame_2.setObjectName("frame_2")
        self.horizontalLayout_3 = QtGui.QHBoxLayout(self.frame_2)
        self.horizontalLayout_3.setContentsMargins(24, -1, 24, -1)
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        self.label_2 = QtGui.QLabel(self.frame_2)
        font = QtGui.QFont()
        font.setPointSize(16)
        font.setWeight(75)
        font.setBold(True)
        self.label_2.setFont(font)
        self.label_2.setStyleSheet("background-color: rgba(255, 255, 255, 0);")
        self.label_2.setObjectName("label_2")
        self.horizontalLayout_3.addWidget(self.label_2)
        self.btnEIPPreferences = QtGui.QPushButton(self.frame_2)
        self.btnEIPPreferences.setMaximumSize(QtCore.QSize(48, 20))
        self.btnEIPPreferences.setStyleSheet("")
        self.btnEIPPreferences.setText("")
        icon1 = QtGui.QIcon()
        icon1.addPixmap(QtGui.QPixmap(":/images/black/32/gear.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.btnEIPPreferences.setIcon(icon1)
        self.btnEIPPreferences.setAutoDefault(False)
        self.btnEIPPreferences.setDefault(False)
        self.btnEIPPreferences.setFlat(False)
        self.btnEIPPreferences.setObjectName("btnEIPPreferences")
        self.horizontalLayout_3.addWidget(self.btnEIPPreferences)
        self.verticalLayout_2.addWidget(self.frame_2)
        self.eipLayout = QtGui.QVBoxLayout()
        self.eipLayout.setContentsMargins(12, 0, 12, 0)
        self.eipLayout.setObjectName("eipLayout")
        self.verticalLayout_2.addLayout(self.eipLayout)
        self.verticalLayout.addWidget(self.eipWidget)
        self.line = QtGui.QFrame(self.scrollAreaWidgetContents)
        self.line.setFrameShape(QtGui.QFrame.HLine)
        self.line.setFrameShadow(QtGui.QFrame.Sunken)
        self.line.setObjectName("line")
        self.verticalLayout.addWidget(self.line)
        self.frame = QtGui.QFrame(self.scrollAreaWidgetContents)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Preferred, QtGui.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.frame.sizePolicy().hasHeightForWidth())
        self.frame.setSizePolicy(sizePolicy)
        self.frame.setAutoFillBackground(False)
        self.frame.setStyleSheet("QFrame{\n"
"background-color: qlineargradient(spread:pad, x1:0, y1:1, x2:0, y2:0, stop:0 rgba(160, 160, 160, 128), stop:1 rgba(255, 255, 255, 0));\n"
"}")
        self.frame.setObjectName("frame")
        self.horizontalLayout = QtGui.QHBoxLayout(self.frame)
        self.horizontalLayout.setContentsMargins(24, -1, 24, -1)
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.lblLoginProvider = QtGui.QLabel(self.frame)
        font = QtGui.QFont()
        font.setPointSize(16)
        font.setWeight(75)
        font.setBold(True)
        self.lblLoginProvider.setFont(font)
        self.lblLoginProvider.setStyleSheet("background-color: rgba(255, 255, 255, 0);")
        self.lblLoginProvider.setObjectName("lblLoginProvider")
        self.horizontalLayout.addWidget(self.lblLoginProvider)
        self.btnPreferences = QtGui.QPushButton(self.frame)
        self.btnPreferences.setMaximumSize(QtCore.QSize(48, 20))
        self.btnPreferences.setStyleSheet("")
        self.btnPreferences.setText("")
        self.btnPreferences.setIcon(icon1)
        self.btnPreferences.setObjectName("btnPreferences")
        self.horizontalLayout.addWidget(self.btnPreferences)
        self.verticalLayout.addWidget(self.frame)
        self.loginLayout = QtGui.QVBoxLayout()
        self.loginLayout.setContentsMargins(-1, 12, -1, -1)
        self.loginLayout.setObjectName("loginLayout")
        self.verticalLayout.addLayout(self.loginLayout)
        self.line_2 = QtGui.QFrame(self.scrollAreaWidgetContents)
        self.line_2.setFrameShape(QtGui.QFrame.HLine)
        self.line_2.setFrameShadow(QtGui.QFrame.Sunken)
        self.line_2.setObjectName("line_2")
        self.verticalLayout.addWidget(self.line_2)
        self.mailWidget = QtGui.QWidget(self.scrollAreaWidgetContents)
        self.mailWidget.setObjectName("mailWidget")
        self.verticalLayout_3 = QtGui.QVBoxLayout(self.mailWidget)
        self.verticalLayout_3.setSpacing(0)
        self.verticalLayout_3.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout_3.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.mailLayout = QtGui.QVBoxLayout()
        self.mailLayout.setSpacing(-1)
        self.mailLayout.setContentsMargins(12, 12, 12, 12)
        self.mailLayout.setObjectName("mailLayout")
        self.verticalLayout_3.addLayout(self.mailLayout)
        self.verticalLayout.addWidget(self.mailWidget)
        spacerItem = QtGui.QSpacerItem(20, 0, QtGui.QSizePolicy.Minimum, QtGui.QSizePolicy.Expanding)
        self.verticalLayout.addItem(spacerItem)
        self.scrollArea.setWidget(self.scrollAreaWidgetContents)
        self.gridLayout.addWidget(self.scrollArea, 1, 0, 1, 2)
        self.gridLayout_4 = QtGui.QGridLayout()
        self.gridLayout_4.setObjectName("gridLayout_4")
        self.lblNewUpdates = QtGui.QLabel(self.centralwidget)
        self.lblNewUpdates.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.lblNewUpdates.setObjectName("lblNewUpdates")
        self.gridLayout_4.addWidget(self.lblNewUpdates, 1, 1, 1, 1)
        spacerItem1 = QtGui.QSpacerItem(40, 0, QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Minimum)
        self.gridLayout_4.addItem(spacerItem1, 1, 0, 1, 1)
        spacerItem2 = QtGui.QSpacerItem(40, 0, QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Minimum)
        self.gridLayout_4.addItem(spacerItem2, 0, 1, 1, 1)
        self.btnMore = QtGui.QPushButton(self.centralwidget)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Preferred, QtGui.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.btnMore.sizePolicy().hasHeightForWidth())
        self.btnMore.setSizePolicy(sizePolicy)
        self.btnMore.setFlat(True)
        self.btnMore.setObjectName("btnMore")
        self.gridLayout_4.addWidget(self.btnMore, 1, 2, 1, 1)
        spacerItem3 = QtGui.QSpacerItem(40, 0, QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Minimum)
        self.gridLayout_4.addItem(spacerItem3, 1, 3, 1, 1)
        self.gridLayout.addLayout(self.gridLayout_4, 0, 0, 1, 2)
        MainWindow.setCentralWidget(self.centralwidget)
        self.menubar = QtGui.QMenuBar(MainWindow)
        self.menubar.setGeometry(QtCore.QRect(0, 0, 524, 22))
        self.menubar.setObjectName("menubar")
        self.menuFile = QtGui.QMenu(self.menubar)
        self.menuFile.setObjectName("menuFile")
        self.menuHelp = QtGui.QMenu(self.menubar)
        self.menuHelp.setObjectName("menuHelp")
        MainWindow.setMenuBar(self.menubar)
        self.statusbar = QtGui.QStatusBar(MainWindow)
        self.statusbar.setObjectName("statusbar")
        MainWindow.setStatusBar(self.statusbar)
        self.action_preferences = QtGui.QAction(MainWindow)
        self.action_preferences.setObjectName("action_preferences")
        self.action_quit = QtGui.QAction(MainWindow)
        self.action_quit.setObjectName("action_quit")
        self.action_about_leap = QtGui.QAction(MainWindow)
        self.action_about_leap.setObjectName("action_about_leap")
        self.action_help = QtGui.QAction(MainWindow)
        self.action_help.setObjectName("action_help")
        self.action_wizard = QtGui.QAction(MainWindow)
        self.action_wizard.setObjectName("action_wizard")
        self.action_show_logs = QtGui.QAction(MainWindow)
        self.action_show_logs.setObjectName("action_show_logs")
        self.action_create_new_account = QtGui.QAction(MainWindow)
        self.action_create_new_account.setObjectName("action_create_new_account")
        self.menuFile.addAction(self.action_create_new_account)
        self.menuFile.addSeparator()
        self.menuFile.addAction(self.action_quit)
        self.menuHelp.addAction(self.action_help)
        self.menuHelp.addAction(self.action_show_logs)
        self.menuHelp.addSeparator()
        self.menuHelp.addAction(self.action_about_leap)
        self.menubar.addAction(self.menuFile.menuAction())
        self.menubar.addAction(self.menuHelp.menuAction())

        self.retranslateUi(MainWindow)
        QtCore.QMetaObject.connectSlotsByName(MainWindow)

    def retranslateUi(self, MainWindow):
        MainWindow.setWindowTitle(QtGui.QApplication.translate("MainWindow", "Bitmask", None, QtGui.QApplication.UnicodeUTF8))
        self.label_2.setText(QtGui.QApplication.translate("MainWindow", "Encrypted Internet", None, QtGui.QApplication.UnicodeUTF8))
        self.lblLoginProvider.setText(QtGui.QApplication.translate("MainWindow", "Login", None, QtGui.QApplication.UnicodeUTF8))
        self.lblNewUpdates.setText(QtGui.QApplication.translate("MainWindow", "There are new updates available, please restart.", None, QtGui.QApplication.UnicodeUTF8))
        self.btnMore.setText(QtGui.QApplication.translate("MainWindow", "More...", None, QtGui.QApplication.UnicodeUTF8))
        self.menuFile.setTitle(QtGui.QApplication.translate("MainWindow", "&Bitmask", None, QtGui.QApplication.UnicodeUTF8))
        self.menuHelp.setTitle(QtGui.QApplication.translate("MainWindow", "Help", None, QtGui.QApplication.UnicodeUTF8))
        self.action_preferences.setText(QtGui.QApplication.translate("MainWindow", "Preferences...", None, QtGui.QApplication.UnicodeUTF8))
        self.action_quit.setText(QtGui.QApplication.translate("MainWindow", "&Quit", None, QtGui.QApplication.UnicodeUTF8))
        self.action_about_leap.setText(QtGui.QApplication.translate("MainWindow", "About &Bitmask", None, QtGui.QApplication.UnicodeUTF8))
        self.action_help.setText(QtGui.QApplication.translate("MainWindow", "&Help", None, QtGui.QApplication.UnicodeUTF8))
        self.action_wizard.setText(QtGui.QApplication.translate("MainWindow", "&Wizard", None, QtGui.QApplication.UnicodeUTF8))
        self.action_show_logs.setText(QtGui.QApplication.translate("MainWindow", "Show &Log", None, QtGui.QApplication.UnicodeUTF8))
        self.action_create_new_account.setText(QtGui.QApplication.translate("MainWindow", "Create a new account...", None, QtGui.QApplication.UnicodeUTF8))

import mainwindow_rc
import locale_rc



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'src/leap/bitmask/gui/ui/wizard.ui'
#
# Created: Wed Oct 30 17:39:23 2013
#      by: pyside-uic 0.2.15 running on PySide 1.2.1
#
# WARNING! All changes made in this file will be lost!

from PySide import QtCore, QtGui

class Ui_Wizard(object):
    def setupUi(self, Wizard):
        Wizard.setObjectName("Wizard")
        Wizard.resize(536, 510)
        Wizard.setMinimumSize(QtCore.QSize(536, 510))
        Wizard.setMaximumSize(QtCore.QSize(536, 510))
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/images/mask-icon.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        Wizard.setWindowIcon(icon)
        Wizard.setModal(True)
        Wizard.setWizardStyle(QtGui.QWizard.ModernStyle)
        Wizard.setOptions(QtGui.QWizard.IndependentPages)
        self.introduction_page = QtGui.QWizardPage()
        self.introduction_page.setObjectName("introduction_page")
        self.gridLayout = QtGui.QGridLayout(self.introduction_page)
        self.gridLayout.setObjectName("gridLayout")
        self.rdoLogin = QtGui.QRadioButton(self.introduction_page)
        self.rdoLogin.setObjectName("rdoLogin")
        self.gridLayout.addWidget(self.rdoLogin, 3, 0, 1, 1)
        self.label_3 = QtGui.QLabel(self.introduction_page)
        self.label_3.setTextFormat(QtCore.Qt.RichText)
        self.label_3.setWordWrap(True)
        self.label_3.setObjectName("label_3")
        self.gridLayout.addWidget(self.label_3, 0, 0, 1, 1)
        self.rdoRegister = QtGui.QRadioButton(self.introduction_page)
        self.rdoRegister.setChecked(True)
        self.rdoRegister.setObjectName("rdoRegister")
        self.gridLayout.addWidget(self.rdoRegister, 2, 0, 1, 1)
        spacerItem = QtGui.QSpacerItem(20, 40, QtGui.QSizePolicy.Minimum, QtGui.QSizePolicy.Expanding)
        self.gridLayout.addItem(spacerItem, 1, 0, 1, 1)
        spacerItem1 = QtGui.QSpacerItem(20, 40, QtGui.QSizePolicy.Minimum, QtGui.QSizePolicy.Expanding)
        self.gridLayout.addItem(spacerItem1, 4, 0, 1, 1)
        Wizard.addPage(self.introduction_page)
        self.select_provider_page = WizardPage()
        self.select_provider_page.setObjectName("select_provider_page")
        self.gridLayout_2 = QtGui.QGridLayout(self.select_provider_page)
        self.gridLayout_2.setObjectName("gridLayout_2")
        spacerItem2 = QtGui.QSpacerItem(20, 60, QtGui.QSizePolicy.Minimum, QtGui.QSizePolicy.Expanding)
        self.gridLayout_2.addItem(spacerItem2, 0, 1, 1, 1)
        spacerItem3 = QtGui.QSpacerItem(20, 40, QtGui.QSizePolicy.Minimum, QtGui.QSizePolicy.Expanding)
        self.gridLayout_2.addItem(spacerItem3, 5, 1, 1, 1)
        self.grpCheckProvider = QtGui.QGroupBox(self.select_provider_page)
        self.grpCheckProvider.setObjectName("grpCheckProvider")
        self.gridLayout_3 = QtGui.QGridLayout(self.grpCheckProvider)
        self.gridLayout_3.setObjectName("gridLayout_3")
        self.label_5 = QtGui.QLabel(self.grpCheckProvider)
        self.label_5.setObjectName("label_5")
        self.gridLayout_3.addWidget(self.label_5, 3, 0, 1, 1)
        self.label_4 = QtGui.QLabel(self.grpCheckProvider)
        self.label_4.setObjectName("label_4")
        self.gridLayout_3.addWidget(self.label_4, 2, 0, 1, 1)
        self.lblProviderInfo = QtGui.QLabel(self.grpCheckProvider)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Fixed, QtGui.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.lblProviderInfo.sizePolicy().hasHeightForWidth())
        self.lblProviderInfo.setSizePolicy(sizePolicy)
        self.lblProviderInfo.setMinimumSize(QtCore.QSize(24, 24))
        self.lblProviderInfo.setText("")
        self.lblProviderInfo.setPixmap(QtGui.QPixmap(":/images/Emblem-question.png"))
        self.lblProviderInfo.setObjectName("lblProviderInfo")
        self.gridLayout_3.addWidget(self.lblProviderInfo, 3, 1, 1, 1)
        self.lblHTTPS = QtGui.QLabel(self.grpCheckProvider)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Fixed, QtGui.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.lblHTTPS.sizePolicy().hasHeightForWidth())
        self.lblHTTPS.setSizePolicy(sizePolicy)
        self.lblHTTPS.setMinimumSize(QtCore.QSize(24, 24))
        self.lblHTTPS.setText("")
        self.lblHTTPS.setPixmap(QtGui.QPixmap(":/images/Emblem-question.png"))
        self.lblHTTPS.setObjectName("lblHTTPS")
        self.gridLayout_3.addWidget(self.lblHTTPS, 2, 1, 1, 1)
        self.lblNameResolution = QtGui.QLabel(self.grpCheckProvider)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Fixed, QtGui.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.lblNameResolution.sizePolicy().hasHeightForWidth())
        self.lblNameResolution.setSizePolicy(sizePolicy)
        self.lblNameResolution.setMinimumSize(QtCore.QSize(24, 24))
        self.lblNameResolution.setText("")
        self.lblNameResolution.setPixmap(QtGui.QPixmap(":/images/Emblem-question.png"))
        self.lblNameResolution.setObjectName("lblNameResolution")
        self.gridLayout_3.addWidget(self.lblNameResolution, 1, 1, 1, 1)
        self.label_2 = QtGui.QLabel(self.grpCheckProvider)
        self.label_2.setObjectName("label_2")
        self.gridLayout_3.addWidget(self.label_2, 1, 0, 1, 1)
        spacerItem4 = QtGui.QSpacerItem(40, 0, QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Minimum)
        self.gridLayout_3.addItem(spacerItem4, 0, 0, 1, 1)
        self.gridLayout_2.addWidget(self.grpCheckProvider, 6, 0, 1, 3)
        self.lblProviderSelectStatus = QtGui.QLabel(self.select_provider_page)
        self.lblProviderSelectStatus.setText("")
        self.lblProviderSelectStatus.setObjectName("lblProviderSelectStatus")
        self.gridLayout_2.addWidget(self.lblProviderSelectStatus, 4, 1, 1, 2)
        self.groupBox = QtGui.QGroupBox(self.select_provider_page)
        self.groupBox.setObjectName("groupBox")
        self.gridLayout_5 = QtGui.QGridLayout(self.groupBox)
        self.gridLayout_5.setObjectName("gridLayout_5")
        self.rbNewProvider = QtGui.QRadioButton(self.groupBox)
        self.rbNewProvider.setChecked(True)
        self.rbNewProvider.setObjectName("rbNewProvider")
        self.gridLayout_5.addWidget(self.rbNewProvider, 0, 0, 1, 1)
        self.rbExistingProvider = QtGui.QRadioButton(self.groupBox)
        self.rbExistingProvider.setObjectName("rbExistingProvider")
        self.gridLayout_5.addWidget(self.rbExistingProvider, 2, 0, 1, 1)
        self.label = QtGui.QLabel(self.groupBox)
        self.label.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.label.setObjectName("label")
        self.gridLayout_5.addWidget(self.label, 1, 0, 1, 1)
        self.lnProvider = QtGui.QLineEdit(self.groupBox)
        self.lnProvider.setObjectName("lnProvider")
        self.gridLayout_5.addWidget(self.lnProvider, 1, 1, 1, 1)
        self.btnCheck = QtGui.QPushButton(self.groupBox)
        self.btnCheck.setObjectName("btnCheck")
        self.gridLayout_5.addWidget(self.btnCheck, 1, 2, 1, 1)
        self.label_8 = QtGui.QLabel(self.groupBox)
        self.label_8.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.label_8.setObjectName("label_8")
        self.gridLayout_5.addWidget(self.label_8, 3, 0, 1, 1)
        self.cbProviders = QtGui.QComboBox(self.groupBox)
        self.cbProviders.setEnabled(False)
        self.cbProviders.setObjectName("cbProviders")
        self.gridLayout_5.addWidget(self.cbProviders, 3, 1, 1, 1)
        self.gridLayout_2.addWidget(self.groupBox, 1, 0, 1, 3)
        Wizard.addPage(self.select_provider_page)
        self.provider_info_page = QtGui.QWizardPage()
        self.provider_info_page.setObjectName("provider_info_page")
        self.gridLayout_4 = QtGui.QGridLayout(self.provider_info_page)
        self.gridLayout_4.setObjectName("gridLayout_4")
        self.lblProviderName = QtGui.QLabel(self.provider_info_page)
        self.lblProviderName.setObjectName("lblProviderName")
        self.gridLayout_4.addWidget(self.lblProviderName, 1, 0, 1, 2)
        spacerItem5 = QtGui.QSpacerItem(20, 40, QtGui.QSizePolicy.Minimum, QtGui.QSizePolicy.Expanding)
        self.gridLayout_4.addItem(spacerItem5, 6, 1, 1, 1)
        self.lblProviderDesc = QtGui.QLabel(self.provider_info_page)
        self.lblProviderDesc.setMinimumSize(QtCore.QSize(0, 0))
        self.lblProviderDesc.setBaseSize(QtCore.QSize(200, 0))
        self.lblProviderDesc.setWordWrap(True)
        self.lblProviderDesc.setObjectName("lblProviderDesc")
        self.gridLayout_4.addWidget(self.lblProviderDesc, 3, 1, 1, 2)
        self.lblServ = QtGui.QLabel(self.provider_info_page)
        self.lblServ.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.lblServ.setObjectName("lblServ")
        self.gridLayout_4.addWidget(self.lblServ, 4, 0, 1, 1)
        self.lblServicesOffered = QtGui.QLabel(self.provider_info_page)
        self.lblServicesOffered.setObjectName("lblServicesOffered")
        self.gridLayout_4.addWidget(self.lblServicesOffered, 4, 1, 1, 1)
        spacerItem6 = QtGui.QSpacerItem(40, 20, QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Minimum)
        self.gridLayout_4.addItem(spacerItem6, 4, 2, 1, 1)
        self.label_12 = QtGui.QLabel(self.provider_info_page)
        self.label_12.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.label_12.setObjectName("label_12")
        self.gridLayout_4.addWidget(self.label_12, 5, 0, 1, 1)
        self.lblProviderPolicy = QtGui.QLabel(self.provider_info_page)
        self.lblProviderPolicy.setObjectName("lblProviderPolicy")
        self.gridLayout_4.addWidget(self.lblProviderPolicy, 5, 1, 1, 1)
        spacerItem7 = QtGui.QSpacerItem(20, 40, QtGui.QSizePolicy.Minimum, QtGui.QSizePolicy.Expanding)
        self.gridLayout_4.addItem(spacerItem7, 0, 1, 1, 1)
        self.label_6 = QtGui.QLabel(self.provider_info_page)
        self.label_6.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.label_6.setObjectName("label_6")
        self.gridLayout_4.addWidget(self.label_6, 2, 0, 1, 1)
        self.lblProviderURL = QtGui.QLabel(self.provider_info_page)
        self.lblProviderURL.setObjectName("lblProviderURL")
        self.gridLayout_4.addWidget(self.lblProviderURL, 2, 1, 1, 2)
        self.label_7 = QtGui.QLabel(self.provider_info_page)
        self.label_7.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTop|QtCore.Qt.AlignTrailing)
        self.label_7.setObjectName("label_7")
        self.gridLayout_4.addWidget(self.label_7, 3, 0, 1, 1)
        Wizard.addPage(self.provider_info_page)
        self.setup_provider_page = WizardPage()
        self.setup_provider_page.setObjectName("setup_provider_page")
        self.verticalLayout = QtGui.QVBoxLayout(self.setup_provider_page)
        self.verticalLayout.setObjectName("verticalLayout")
        spacerItem8 = QtGui.QSpacerItem(20, 60, QtGui.QSizePolicy.Minimum, QtGui.QSizePolicy.Expanding)
        self.verticalLayout.addItem(spacerItem8)
        self.lblSetupProviderExpl = QtGui.QLabel(self.setup_provider_page)
        self.lblSetupProviderExpl.setWordWrap(True)
        self.lblSetupProviderExpl.setObjectName("lblSetupProviderExpl")
        self.verticalLayout.addWidget(self.lblSetupProviderExpl)
        spacerItem9 = QtGui.QSpacerItem(20, 40, QtGui.QSizePolicy.Minimum, QtGui.QSizePolicy.Expanding)
        self.verticalLayout.addItem(spacerItem9)
        self.groupBox_2 = QtGui.QGroupBox(self.setup_provider_page)
        self.groupBox_2.setObjectName("groupBox_2")
        self.gridLayout_6 = QtGui.QGridLayout(self.groupBox_2)
        self.gridLayout_6.setObjectName("gridLayout_6")
        self.lblCheckCaFpr = QtGui.QLabel(self.groupBox_2)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Fixed, QtGui.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.lblCheckCaFpr.sizePolicy().hasHeightForWidth())
        self.lblCheckCaFpr.setSizePolicy(sizePolicy)
        self.lblCheckCaFpr.setMinimumSize(QtCore.QSize(24, 24))
        self.lblCheckCaFpr.setText("")
        self.lblCheckCaFpr.setPixmap(QtGui.QPixmap(":/images/Emblem-question.png"))
        self.lblCheckCaFpr.setObjectName("lblCheckCaFpr")
        self.gridLayout_6.addWidget(self.lblCheckCaFpr, 2, 1, 1, 1)
        self.lblDownloadCaCert = QtGui.QLabel(self.groupBox_2)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Fixed, QtGui.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.lblDownloadCaCert.sizePolicy().hasHeightForWidth())
        self.lblDownloadCaCert.setSizePolicy(sizePolicy)
        self.lblDownloadCaCert.setMinimumSize(QtCore.QSize(24, 24))
        self.lblDownloadCaCert.setText("")
        self.lblDownloadCaCert.setPixmap(QtGui.QPixmap(":/images/Emblem-question.png"))
        self.lblDownloadCaCert.setObjectName("lblDownloadCaCert")
        self.gridLayout_6.addWidget(self.lblDownloadCaCert, 1, 1, 1, 1)
        self.label_9 = QtGui.QLabel(self.groupBox_2)
        self.label_9.setObjectName("label_9")
        self.gridLayout_6.addWidget(self.label_9, 1, 0, 1, 1)
        self.label_10 = QtGui.QLabel(self.groupBox_2)
        self.label_10.setObjectName("label_10")
        self.gridLayout_6.addWidget(self.label_10, 2, 0, 1, 1)
        self.label_11 = QtGui.QLabel(self.groupBox_2)
        self.label_11.setObjectName("label_11")
        self.gridLayout_6.addWidget(self.label_11, 3, 0, 1, 1)
        self.lblCheckApiCert = QtGui.QLabel(self.groupBox_2)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Fixed, QtGui.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.lblCheckApiCert.sizePolicy().hasHeightForWidth())
        self.lblCheckApiCert.setSizePolicy(sizePolicy)
        self.lblCheckApiCert.setMinimumSize(QtCore.QSize(24, 24))
        self.lblCheckApiCert.setText("")
        self.lblCheckApiCert.setPixmap(QtGui.QPixmap(":/images/Emblem-question.png"))
        self.lblCheckApiCert.setObjectName("lblCheckApiCert")
        self.gridLayout_6.addWidget(self.lblCheckApiCert, 3, 1, 1, 1)
        spacerItem10 = QtGui.QSpacerItem(40, 0, QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Minimum)
        self.gridLayout_6.addItem(spacerItem10, 0, 0, 1, 1)
        self.verticalLayout.addWidget(self.groupBox_2)
        spacerItem11 = QtGui.QSpacerItem(20, 40, QtGui.QSizePolicy.Minimum, QtGui.QSizePolicy.Expanding)
        self.verticalLayout.addItem(spacerItem11)
        Wizard.addPage(self.setup_provider_page)
        self.register_user_page = WizardPage()
        self.register_user_page.setObjectName("register_user_page")
        self.gridLayout_7 = QtGui.QGridLayout(self.register_user_page)
        self.gridLayout_7.setSizeConstraint(QtGui.QLayout.SetDefaultConstraint)
        self.gridLayout_7.setContentsMargins(4, -1, -1, -1)
        self.gridLayout_7.setObjectName("gridLayout_7")
        self.label_16 = QtGui.QLabel(self.register_user_page)
        self.label_16.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.label_16.setObjectName("label_16")
        self.gridLayout_7.addWidget(self.label_16, 3, 0, 1, 1)
        self.lblPassword = QtGui.QLineEdit(self.register_user_page)
        self.lblPassword.setEchoMode(QtGui.QLineEdit.Password)
        self.lblPassword.setObjectName("lblPassword")
        self.gridLayout_7.addWidget(self.lblPassword, 3, 1, 1, 2)
        self.lblPassword2 = QtGui.QLineEdit(self.register_user_page)
        self.lblPassword2.setEchoMode(QtGui.QLineEdit.Password)
        self.lblPassword2.setObjectName("lblPassword2")
        self.gridLayout_7.addWidget(self.lblPassword2, 4, 1, 1, 2)
        self.lblUser = QtGui.QLineEdit(self.register_user_page)
        self.lblUser.setObjectName("lblUser")
        self.gridLayout_7.addWidget(self.lblUser, 2, 1, 1, 2)
        self.label_17 = QtGui.QLabel(self.register_user_page)
        self.label_17.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.label_17.setObjectName("label_17")
        self.gridLayout_7.addWidget(self.label_17, 4, 0, 1, 1)
        self.btnRegister = QtGui.QPushButton(self.register_user_page)
        self.btnRegister.setObjectName("btnRegister")
        self.gridLayout_7.addWidget(self.btnRegister, 6, 1, 1, 1)
        spacerItem12 = QtGui.QSpacerItem(20, 60, QtGui.QSizePolicy.Minimum, QtGui.QSizePolicy.Expanding)
        self.gridLayout_7.addItem(spacerItem12, 0, 1, 1, 1)
        spacerItem13 = QtGui.QSpacerItem(20, 40, QtGui.QSizePolicy.Minimum, QtGui.QSizePolicy.Expanding)
        self.gridLayout_7.addItem(spacerItem13, 7, 1, 1, 1)
        self.label_15 = QtGui.QLabel(self.register_user_page)
        self.label_15.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.label_15.setObjectName("label_15")
        self.gridLayout_7.addWidget(self.label_15, 2, 0, 1, 1)
        self.chkRemember = QtGui.QCheckBox(self.register_user_page)
        self.chkRemember.setEnabled(False)
        self.chkRemember.setObjectName("chkRemember")
        self.gridLayout_7.addWidget(self.chkRemember, 5, 1, 1, 2)
        self.lblRegisterStatus = QtGui.QLabel(self.register_user_page)
        self.lblRegisterStatus.setText("")
        self.lblRegisterStatus.setTextFormat(QtCore.Qt.AutoText)
        self.lblRegisterStatus.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.lblRegisterStatus.setWordWrap(True)
        self.lblRegisterStatus.setObjectName("lblRegisterStatus")
        self.gridLayout_7.addWidget(self.lblRegisterStatus, 1, 1, 1, 2)
        Wizard.addPage(self.register_user_page)
        self.service_selection = QtGui.QWizardPage()
        self.service_selection.setObjectName("service_selection")
        self.gridLayout_8 = QtGui.QGridLayout(self.service_selection)
        self.gridLayout_8.setObjectName("gridLayout_8")
        self.grpServices = QtGui.QGroupBox(self.service_selection)
        self.grpServices.setTitle("Services by PROVIDER")
        self.grpServices.setObjectName("grpServices")
        self.gridLayout_9 = QtGui.QGridLayout(self.grpServices)
        self.gridLayout_9.setObjectName("gridLayout_9")
        self.serviceListLayout = QtGui.QVBoxLayout()
        self.serviceListLayout.setObjectName("serviceListLayout")
        self.gridLayout_9.addLayout(self.serviceListLayout, 0, 0, 1, 1)
        self.gridLayout_8.addWidget(self.grpServices, 0, 0, 1, 1)
        Wizard.addPage(self.service_selection)

        self.retranslateUi(Wizard)
        QtCore.QObject.connect(self.rbExistingProvider, QtCore.SIGNAL("toggled(bool)"), self.cbProviders.setFocus)
        QtCore.QObject.connect(self.rbNewProvider, QtCore.SIGNAL("toggled(bool)"), self.lnProvider.setFocus)
        QtCore.QObject.connect(self.rbExistingProvider, QtCore.SIGNAL("toggled(bool)"), self.lnProvider.setDisabled)
        QtCore.QObject.connect(self.rbNewProvider, QtCore.SIGNAL("toggled(bool)"), self.cbProviders.setDisabled)
        QtCore.QObject.connect(self.rbExistingProvider, QtCore.SIGNAL("toggled(bool)"), self.btnCheck.setDisabled)
        QtCore.QMetaObject.connectSlotsByName(Wizard)
        Wizard.setTabOrder(self.lblUser, self.lblPassword)
        Wizard.setTabOrder(self.lblPassword, self.lblPassword2)
        Wizard.setTabOrder(self.lblPassword2, self.btnRegister)
        Wizard.setTabOrder(self.btnRegister, self.rdoRegister)
        Wizard.setTabOrder(self.rdoRegister, self.rdoLogin)

    def retranslateUi(self, Wizard):
        Wizard.setWindowTitle(QtGui.QApplication.translate("Wizard", "Bitmask first run", None, QtGui.QApplication.UnicodeUTF8))
        self.introduction_page.setTitle(QtGui.QApplication.translate("Wizard", "Welcome", None, QtGui.QApplication.UnicodeUTF8))
        self.introduction_page.setSubTitle(QtGui.QApplication.translate("Wizard", "This is the Bitmask first run wizard", None, QtGui.QApplication.UnicodeUTF8))
        self.rdoLogin.setText(QtGui.QApplication.translate("Wizard", "Log In with my credentials", None, QtGui.QApplication.UnicodeUTF8))
        self.label_3.setText(QtGui.QApplication.translate("Wizard", "<html><head/><body><p>Now we will guide you through some configuration that is needed before you can connect for the first time.</p><p>If you ever need to modify these options again, you can find the wizard in the <span style=\" font-style:italic;\">\'Settings\'</span> menu from the main window.</p><p>Do you want to <span style=\" font-weight:600;\">sign up</span> for a new account, or <span style=\" font-weight:600;\">log in</span> with an already existing username?</p></body></html>", None, QtGui.QApplication.UnicodeUTF8))
        self.rdoRegister.setText(QtGui.QApplication.translate("Wizard", "Sign up for a new account", None, QtGui.QApplication.UnicodeUTF8))
        self.select_provider_page.setTitle(QtGui.QApplication.translate("Wizard", "Provider selection", None, QtGui.QApplication.UnicodeUTF8))
        self.select_provider_page.setSubTitle(QtGui.QApplication.translate("Wizard", "Please enter the domain of the provider you want to use for your connection", None, QtGui.QApplication.UnicodeUTF8))
        self.grpCheckProvider.setTitle(QtGui.QApplication.translate("Wizard", "Checking for a valid provider", None, QtGui.QApplication.UnicodeUTF8))
        self.label_5.setText(QtGui.QApplication.translate("Wizard", "Getting provider information", None, QtGui.QApplication.UnicodeUTF8))
        self.label_4.setText(QtGui.QApplication.translate("Wizard", "Can we establish a secure connection?", None, QtGui.QApplication.UnicodeUTF8))
        self.label_2.setText(QtGui.QApplication.translate("Wizard", "Can we reach this provider?", None, QtGui.QApplication.UnicodeUTF8))
        self.groupBox.setTitle(QtGui.QApplication.translate("Wizard", "Configure or select a provider", None, QtGui.QApplication.UnicodeUTF8))
        self.rbNewProvider.setText(QtGui.QApplication.translate("Wizard", "Configure new provider:", None, QtGui.QApplication.UnicodeUTF8))
        self.rbExistingProvider.setText(QtGui.QApplication.translate("Wizard", "Use existing one:", None, QtGui.QApplication.UnicodeUTF8))
        self.label.setText(QtGui.QApplication.translate("Wizard", "https://", None, QtGui.QApplication.UnicodeUTF8))
        self.btnCheck.setText(QtGui.QApplication.translate("Wizard", "Check", None, QtGui.QApplication.UnicodeUTF8))
        self.label_8.setText(QtGui.QApplication.translate("Wizard", "https://", None, QtGui.QApplication.UnicodeUTF8))
        self.provider_info_page.setTitle(QtGui.QApplication.translate("Wizard", "Provider Information", None, QtGui.QApplication.UnicodeUTF8))
        self.provider_info_page.setSubTitle(QtGui.QApplication.translate("Wizard", "Description of services offered by this provider", None, QtGui.QApplication.UnicodeUTF8))
        self.lblProviderName.setText(QtGui.QApplication.translate("Wizard", "Name", None, QtGui.QApplication.UnicodeUTF8))
        self.lblProviderDesc.setText(QtGui.QApplication.translate("Wizard", "Desc", None, QtGui.QApplication.UnicodeUTF8))
        self.lblServ.setText(QtGui.QApplication.translate("Wizard", "<b>Services offered:</b>", None, QtGui.QApplication.UnicodeUTF8))
        self.lblServicesOffered.setText(QtGui.QApplication.translate("Wizard", "services", None, QtGui.QApplication.UnicodeUTF8))
        self.label_12.setText(QtGui.QApplication.translate("Wizard", "<b>Enrollment policy:</b>", None, QtGui.QApplication.UnicodeUTF8))
        self.lblProviderPolicy.setText(QtGui.QApplication.translate("Wizard", "policy", None, QtGui.QApplication.UnicodeUTF8))
        self.label_6.setText(QtGui.QApplication.translate("Wizard", "<b>URL:</b>", None, QtGui.QApplication.UnicodeUTF8))
        self.lblProviderURL.setText(QtGui.QApplication.translate("Wizard", "URL", None, QtGui.QApplication.UnicodeUTF8))
        self.label_7.setText(QtGui.QApplication.translate("Wizard", "<b>Description:</b>", None, QtGui.QApplication.UnicodeUTF8))
        self.setup_provider_page.setTitle(QtGui.QApplication.translate("Wizard", "Provider setup", None, QtGui.QApplication.UnicodeUTF8))
        self.setup_provider_page.setSubTitle(QtGui.QApplication.translate("Wizard", "Gathering configuration options for this provider", None, QtGui.QApplication.UnicodeUTF8))
        self.lblSetupProviderExpl.setText(QtGui.QApplication.translate("Wizard", "We are downloading some bits that we need to establish a secure connection with the provider for the first time.", None, QtGui.QApplication.UnicodeUTF8))
        self.groupBox_2.setTitle(QtGui.QApplication.translate("Wizard", "Setting up provider", None, QtGui.QApplication.UnicodeUTF8))
        self.label_9.setText(QtGui.QApplication.translate("Wizard", "Getting info from the Certificate Authority", None, QtGui.QApplication.UnicodeUTF8))
        self.label_10.setText(QtGui.QApplication.translate("Wizard", "Do we trust this Certificate Authority?", None, QtGui.QApplication.UnicodeUTF8))
        self.label_11.setText(QtGui.QApplication.translate("Wizard", "Establishing a trust relationship with this provider", None, QtGui.QApplication.UnicodeUTF8))
        self.register_user_page.setTitle(QtGui.QApplication.translate("Wizard", "Register new user", None, QtGui.QApplication.UnicodeUTF8))
        self.register_user_page.setSubTitle(QtGui.QApplication.translate("Wizard", "Register a new user with provider", None, QtGui.QApplication.UnicodeUTF8))
        self.label_16.setText(QtGui.QApplication.translate("Wizard", "<b>Password:</b>", None, QtGui.QApplication.UnicodeUTF8))
        self.label_17.setText(QtGui.QApplication.translate("Wizard", "<b>Re-enter password:</b>", None, QtGui.QApplication.UnicodeUTF8))
        self.btnRegister.setText(QtGui.QApplication.translate("Wizard", "Register", None, QtGui.QApplication.UnicodeUTF8))
        self.label_15.setText(QtGui.QApplication.translate("Wizard", "<b>Username:</b>", None, QtGui.QApplication.UnicodeUTF8))
        self.chkRemember.setText(QtGui.QApplication.translate("Wizard", "Remember my username and password", None, QtGui.QApplication.UnicodeUTF8))
        self.service_selection.setTitle(QtGui.QApplication.translate("Wizard", "Service selection", None, QtGui.QApplication.UnicodeUTF8))
        self.service_selection.setSubTitle(QtGui.QApplication.translate("Wizard", "Please select the services you would like to have", None, QtGui.QApplication.UnicodeUTF8))

from wizardpage import WizardPage
import mainwindow_rc
import locale_rc



# -*- coding: utf-8 -*-
# soledadconfig.py
# Copyright (C) 2013 LEAP
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""
Soledad configuration
"""
import logging

from leap.bitmask.services import ServiceConfig
from leap.bitmask.services.soledad.soledadspec import get_schema

logger = logging.getLogger(__name__)


class SoledadConfig(ServiceConfig):
    """
    Soledad configuration abstraction class
    """
    _service_name = "soledad"

    def __init__(self):
        ServiceConfig.__init__(self)

    def _get_schema(self):
        """
        Returns the schema corresponding to the version given.

        :rtype: dict or None if the version is not supported.
        """
        return get_schema(self._api_version)

    def get_hosts(self):
        return self._safe_get_value("hosts")

    def get_locations(self):
        return self._safe_get_value("locations")



# -*- coding: utf-8 -*-
# flags.py
# Copyright (C) 2013 LEAP
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""
This file is meant to be used to store global flags that affect the
application.

WARNING: You should NOT use this kind of flags unless you're sure of what
         you're doing, and someone else tells you that you're right.
         Most of the times there is a better and safer alternative.
"""

# The STANDALONE flag is used to:
#   - define a different set of messages for the application when is running
#     inside of a bundle or installed system wide.
#   - use a relative or system wide path to find the configuration files.
#   - search for binaries inside the bundled app instead of the system ones.
#     e.g.: openvpn, gpg
STANDALONE = False



# -*- coding: utf-8 -*-
# wizardpage.py
# Copyright (C) 2013 LEAP
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from PySide import QtGui


class WizardPage(QtGui.QWizardPage):
    """
    Simple wizard page helper
    """

    def __init__(self):
        QtGui.QWizardPage.__init__(self)
        self._completed = False

    def set_completed(self, val=True):
        self._completed = val
        if val:
            self.completeChanged.emit()

    def isComplete(self):
        return self._completed

    def cleanupPage(self):
        self._completed = False
        QtGui.QWizardPage.cleanupPage(self)
